# -*- coding: utf-8 -*-
#########################################################################
## rdesigneur0_5.py ---
## This program is part of 'MOOSE', the
## Messaging Object Oriented Simulation Environment.
##           Copyright (C) 2014 Upinder S. Bhalla. and NCBS
## It is made available under the terms of the
## GNU General Public License version 2 or later.
## See the file COPYING.LIB for the full notice.
#########################################################################

##########################################################################
## This class builds models of
## Reaction-Diffusion and Electrical SIGnaling in NEURons.
## It loads in neuronal and chemical signaling models and embeds the
## latter in the former, including mapping entities like calcium and
## channel conductances, between them.
##########################################################################
from __future__ import print_function, absolute_import, division

# FIXME: Deprecated since 3.4
import imp

import os
import moose
import numpy as np
import math
import sys
import time
import matplotlib.pyplot as plt

import rdesigneur.rmoogli as rmoogli
from rdesigneur.rdesigneurProtos import *
import moose.fixXreacs as fixXreacs

from moose.neuroml.NeuroML import NeuroML
from moose.neuroml.ChannelML import ChannelML

# In python3, cElementTree is deprecated. We do not plan to support python <2.7
# in future, so other imports have been removed.
try:
  from lxml import etree
except ImportError:
  import xml.etree.ElementTree as etree

import csv

#EREST_ACT = -70e-3

def _profile(func):
    """
    Can be used to profile a function. Useful in debugging and profiling.
    Author: Dilawar Singh
    """
    def wrap(self=None, *args, **kwargs):
        t0 = time.time()
        result = func(self, *args, **kwargs)
        print("[INFO ] Took %s sec" % (time.time()-t0))
        return result
    return wrap
        

class BuildError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

#######################################################################

class rdesigneur:
    """The rdesigneur class is used to build models incorporating
    reaction-diffusion and electrical signaling in neurons.
    Params:
        useGssa: True/False               for GSSA in spine and PSD
        combineSegments: True/False       for NeuroML models
        diffusionLength: default 2e-6
        adaptCa: [( Ca_wildcard_string, chem_wildcard_string, offset, scale ),...]
        adaptChem: [( Chem_wildcard_string, elec_wildcard_string, offset, scale ),...]

    I need to put the extra channels now into the NeuroML definition.
    """
    ################################################################
    def __init__(self,
            modelPath = '/model',
            turnOffElec = False,
            useGssa = True,
            combineSegments = True,
            stealCellFromLibrary = False,
            verbose = True,
            benchmark = False,
            addSomaChemCompt = False,  # Put a soma chemCompt on neuroMesh
            addEndoChemCompt = False,  # Put an endo compartment, typically for ER, on each of the NeuroMesh compartments.
            diffusionLength= 2e-6,
            meshLambda = -1.0,    #This is a backward compatibility hack
            temperature = 32,
            chemDt= 0.1,            # Much finer than MOOSE, for multiscale
            diffDt= 0.01,           # 10x finer than MOOSE, for multiscale
            elecDt= 50e-6,          # Same default as from MOOSE
            chemPlotDt = 1.0,       # Same default as from MOOSE
            elecPlotDt = 0.1e-3,    # Same default as from MOOSE
            funcDt = 0.1e-3,        # Used when turnOffElec is False.
                                    # Otherwise system uses chemDt.
            numWaveFrames = 100,    # Number of frames to use for waveplots
            cellProto = [],
            spineProto = [],
            chanProto = [],
            chemProto = [],
            passiveDistrib= [],
            spineDistrib= [],
            chanDistrib = [],
            chemDistrib = [],
            adaptorList= [],
            stimList = [],
            plotList = [],  # elecpath, geom_expr, object, field, title ['wave' [min max]]
            moogList = [], 
            ode_method = "gsl",  # gsl, lsoda, gssa, gillespie
            params = None
        ):
        """ Constructor of the rdesigner. This just sets up internal fields
            for the model building, it doesn't actually create any objects.
        """

        self.modelPath = modelPath
        self.turnOffElec = turnOffElec
        self.useGssa = useGssa
        self.ode_method = ode_method
        self.combineSegments = combineSegments
        self.stealCellFromLibrary = stealCellFromLibrary
        self.verbose = verbose
        self.benchmark = benchmark
        self.addSomaChemCompt = addSomaChemCompt
        self.addEndoChemCompt = addEndoChemCompt
        self.diffusionLength= diffusionLength
        if meshLambda > 0.0:
            print("Warning: meshLambda argument is deprecated. Please use "
                    "'diffusionLength' instead.\nFor now rdesigneur will "
                    "accept this argument.")
            self.diffusionLength = meshLambda
        self.temperature = temperature
        self.chemDt= chemDt
        self.diffDt= diffDt
        self.elecDt= elecDt
        self.elecPlotDt= elecPlotDt
        self.funcDt= funcDt
        self.chemPlotDt= chemPlotDt
        self.numWaveFrames = numWaveFrames

        self.cellProtoList = cellProto
        self.spineProtoList = spineProto
        self.chanProtoList = chanProto
        self.chemProtoList = chemProto

        self.passiveDistrib = passiveDistrib
        self.spineDistrib = spineDistrib
        self.chanDistrib = chanDistrib
        self.chemDistrib = chemDistrib

        self.params = params

        self.adaptorList = adaptorList
        try:
            self.stimList = [ rstim.convertArg(i) for i in stimList ]
            self.plotList = [ rplot.convertArg(i) for i in plotList ]
            self.moogList = [ rmoog.convertArg(i) for i in moogList ]
        except BuildError as msg:
            print("Error: rdesigneur: " + msg)
            quit()

        self.saveAs = []
        self.plotNames = []
        self.wavePlotNames = []
        self.saveNames = []
        self.moogNames = []
        self.cellPortionElist = []
        self.spineComptElist = []
        self.tabForXML = []
        self._endos = []

        if not moose.exists( '/library' ):
            library = moose.Neutral( '/library' )
        try:
            self.buildCellProto()
            self.buildChanProto()
            self.buildSpineProto()
            self.buildChemProto()
        except BuildError as msg:
            print("Error: rdesigneur: Prototype build failed:", msg)
            quit()


    ################################################################
    def _printModelStats( self ):
        if not self.verbose:
            return
        print("Rdesigneur: Elec model has",
            self.elecid.numCompartments, "compartments and",
            self.elecid.numSpines, "spines on",
            len( self.cellPortionElist ), "compartments.")
        if hasattr( self , 'chemid') and len( self.chemDistrib ) > 0:
            #  dmstoich = moose.element( self.dendCompt.path + '/stoich' )
            print("    Chem part of model has the following compartments: ")
            for j in moose.wildcardFind( '/model/chem/##[ISA=ChemCompt]'):
                s = moose.element( j.path + '/stoich' )
                print( "    | In {}, {} voxels X {} pools".format( j.name, j.mesh.num, s.numAllPools ) )

    def buildModel( self, modelPath = '/model' ):
        if moose.exists( modelPath ):
            print("rdesigneur::buildModel: Build failed. Model '",
                modelPath, "' already exists.")
            return
        self.model = moose.Neutral( modelPath )
        self.modelPath = modelPath
        funcs = [self.installCellFromProtos, self.buildPassiveDistrib
            , self.buildChanDistrib, self.buildSpineDistrib, self.buildChemDistrib
            , self._configureSolvers, self.buildAdaptors, self._buildStims
            , self._buildPlots, self._buildMoogli, self._configureHSolve
            , self._configureClocks, self._printModelStats]

        for i, _func in enumerate(funcs):
            if self.benchmark:
                print("- (%02d/%d) Executing %25s"%(i+1, len(funcs), _func.__name__), end=' ' )
            t0 = time.time()
            try:
                _func()
            except BuildError as msg:
                print("Error: rdesigneur: model build failed:", msg)
                moose.delete(self.model)
                return
            t = time.time() - t0
            if self.benchmark:
                msg = r'    ... DONE'
                if t > 0.1:
                    msg += ' %.3f sec' % t
                print(msg)
            sys.stdout.flush()

    def installCellFromProtos( self ):
        if self.stealCellFromLibrary:
            moose.move( self.elecid, self.model )
            if self.elecid.name != 'elec':
                self.elecid.name = 'elec'
        else:
            moose.copy( self.elecid, self.model, 'elec' )
            self.elecid = moose.element( self.model.path + '/elec' )
            self.elecid.buildSegmentTree() # rebuild: copy has happened.
        if hasattr( self, 'chemid' ):
            self.validateChem()
            if self.stealCellFromLibrary:
                moose.move( self.chemid, self.model )
                if self.chemid.name != 'chem':
                    self.chemid.name = 'chem'
            else:
                moose.copy( self.chemid, self.model, 'chem' )
                self.chemid = moose.element( self.model.path + '/chem' )

        ep = self.elecid.path
        somaList = moose.wildcardFind( ep + '/#oma#[ISA=CompartmentBase]' )
        if len( somaList ) == 0:
            somaList = moose.wildcardFind( ep + '/#[ISA=CompartmentBase]' )
        if len( somaList ) == 0:
            raise BuildError( "installCellFromProto: No soma found" )
        maxdia = 0.0
        for i in somaList:
            if ( i.diameter > maxdia ):
                self.soma = i

    ################################################################
    # Some utility functions for building prototypes.
    ################################################################
    # Return true if it is a function.
    def buildProtoFromFunction( self, func, protoName ):
        bracePos = func.find( '()' )
        if bracePos == -1:
            return False

        # . can be in path name as well. Find the last dot which is most likely
        # to be the function name.
        modPos = func.rfind( "." )
        if ( modPos != -1 ): # Function is in a file, load and check
            resolvedPath = os.path.realpath( func[0:modPos] )
            pathTokens = resolvedPath.split('/')
            pathTokens = ['/'] + pathTokens
            modulePath = os.path.realpath(os.path.join(*pathTokens[:-1]))
            moduleName = pathTokens[-1]
            funcName = func[modPos+1:bracePos]
            moduleFile, pathName, description = imp.find_module(moduleName, [modulePath])
            try:
                module = imp.load_module(moduleName, moduleFile, pathName, description)
                funcObj = getattr(module, funcName)
                funcObj(protoName)
                return True
            finally:
                moduleFile.close()
            return False
        if not func[0:bracePos] in globals():
            raise BuildError( \
                protoName + " Proto: global function '" +func+"' not known.")
        globals().get( func[0:bracePos] )( protoName )
        return True

    # Class or file options. True if extension is found in
    def isKnownClassOrFile( self, name, suffices ):
        for i in suffices:
            if name.rfind( '.'+i ) >= 0 :
                return True
        return False



    # Checks all protos, builds them and return true. If it was a file
    # then it has to return false and invite the calling function to build
    # If it fails then the exception takes over.
    def checkAndBuildProto( self, protoType, protoVec, knownClasses, knownFileTypes ):
        if len(protoVec) != 2:
            raise BuildError( \
                protoType + "Proto: nargs should be 2, is " + \
                    str( len(protoVec)  ))
        if moose.exists( '/library/' + protoVec[1] ):
            # Assume the job is already done, just skip it.
            return True
            '''
            raise BuildError( \
                protoType + "Proto: object /library/" + \
                    protoVec[1] + " already exists." )
            '''
        # Check and build the proto from a class name
        if protoVec[0][:5] == 'moose':
            protoName = protoVec[0][6:]
            if self.isKnownClassOrFile( protoName, knownClasses ):
                try:
                    getattr( moose, protoName )( '/library/' + protoVec[1] )
                except AttributeError:
                    raise BuildError( protoType + "Proto: Moose class '" \
                            + protoVec[0] + "' not found." )
                return True

        if self.buildProtoFromFunction( protoVec[0], protoVec[1] ):
            return True
        # Maybe the proto is already in memory
        # Avoid relative file paths going toward root
        if protoVec[0][:3] != "../":
            if moose.exists( protoVec[0] ):
                moose.copy( protoVec[0], '/library/' + protoVec[1] )
                return True
            if moose.exists( '/library/' + protoVec[0] ):
                #moose.copy('/library/' + protoVec[0], '/library/', protoVec[1])
                if self.verbose:
                    print('renaming /library/' + protoVec[0] + ' to ' + protoVec[1])
                moose.element( '/library/' + protoVec[0]).name = protoVec[1]
                #moose.le( '/library' )
                return True
        # Check if there is a matching suffix for file type.
        if self.isKnownClassOrFile( protoVec[0], knownFileTypes ):
            return False
        else:
            raise BuildError( \
                protoType + "Proto: File type '" + protoVec[0] + \
                "' not known." )
        return True

    ################################################################
    # Here are the functions to build the type-specific prototypes.
    ################################################################
    def buildCellProto( self ):
        # cellProtoList args:
        # Option 1: zero args: make standard soma, len and dia 500 um.
        # Option 2: [name, library_proto_name]: uses library proto
        # Option 3: [fname.suffix, cellname ]: Loads cell from file
        # Option 4: [moose<Classname>, cellname]: Makes proto of class
        # Option 5: [funcname, cellname]: Calls named function with specified name of cell to be made.
        # Option 6: [path, cellname]: Copies path to library as proto
        # Option 7: [libraryName, cellname]: Renames library entry as proto
        # Below two options only need the first two args, rest are optional
        # Defailt values are given.
        # Option 8: [somaProto,name, somaDia=5e-4, somaLen=5e-4]
        # Option 9: [ballAndStick,name, somaDia=10e-6, somaLen=10e-6, 
        #       dendDia=4e-6, dendLen=200e-6, numDendSeg=1]
        if len( self.cellProtoList ) == 0:
            ''' Make HH squid model sized compartment:
            len and dia 500 microns. CM = 0.01 F/m^2, RA =
            '''
            self.elecid = makePassiveHHsoma( name = 'cell' )
            assert( moose.exists( '/library/cell/soma' ) )
            self.soma = moose.element( '/library/cell/soma' )
            return

            '''
            self.elecid = moose.Neuron( '/library/cell' )
            dia = 500e-6
            self.soma = buildCompt( self.elecid, 'soma', dia, dia, 0.0,
                0.33333333, 3000, 0.01 )
            self.soma.initVm = -65e-3 # Resting of -65, from HH
            self.soma.Em = -54.4e-3 # 10.6 mV above resting of -65, from HH
            '''

        for i in self.cellProtoList:
            if i[0] == 'somaProto':
                self._buildElecSoma( i )
            elif i[0] == 'ballAndStick':
                self._buildElecBallAndStick( i )
            elif self.checkAndBuildProto( "cell", i, \
                ["Compartment", "SymCompartment"], ["swc", "p", "nml", "xml"] ):
                self.elecid = moose.element( '/library/' + i[1] )
            else:
                self._loadElec( i[0], i[1] )
            self.elecid.buildSegmentTree()

    def buildSpineProto( self ):
        for i in self.spineProtoList:
            if not self.checkAndBuildProto( "spine", i, \
                ["Compartment", "SymCompartment"], ["swc", "p", "nml", "xml"] ):
                self._loadElec( i[0], i[1] )

    def parseChanName( self, name ):
        if name[-4:] == ".xml":
            period = name.rfind( '.' )
            slash = name.rfind( '/' )
            if ( slash >= period ):
                raise BuildError( "chanProto: bad filename:" + name )
            if ( slash < 0 ):
                return name[:period]
            else:
                return name[slash+1:period]

    def buildChanProto( self ):
        for i in self.chanProtoList:
            if len(i) == 1:
                chanName = self.parseChanName( i[0] )
            else:
                chanName = i[1]
            j = [i[0], chanName]
            if not self.checkAndBuildProto( "chan", j, [], ["xml"] ):
                cm = ChannelML( {'temperature': self.temperature} )
                cm.readChannelMLFromFile( i[0] )
                if ( len( i ) == 2 ):
                    chan = moose.element( '/library/' + chanName )
                    chan.name = i[1]

    def buildChemProto( self ):
        for i in self.chemProtoList:
            if not self.checkAndBuildProto( "chem", i, \
                ["Pool"], ["g", "sbml", "xml" ] ):
                self._loadChem( i[0], i[1] )
            self.chemid = moose.element( '/library/' + i[1] )
    ################################################################
    def _buildElecSoma( self, args ):
        parms = [ 'somaProto', 'soma', 5e-4, 5e-4 ] # somaDia, somaLen
        for i in range( len(args) ):
            parms[i] = args[i]
        cell = moose.Neuron( '/library/' + parms[1] )
        buildCompt( cell, 'soma', dia = parms[2], dx = parms[3] )
        self.elecid = cell
        return cell
        
    ################################################################
    def _buildElecBallAndStick( self, args ):
        parms = [ 'ballAndStick', 'cell', 10e-6, 10e-6, 4e-6, 200e-6, 1 ] # somaDia, somaLen, dendDia, dendLen, dendNumSeg
        for i in range( len(args) ):
            parms[i] = args[i]
        if parms[6] <= 0:
            return self.buildElecSoma( parms[:4] )
        cell = moose.Neuron( '/library/' + parms[1] )
        prev = buildCompt( cell, 'soma', dia = args[2], dx = args[3] )
        dx = parms[5]/parms[6]
        x = prev.x
        for i in range( parms[6] ):
            compt = buildCompt( cell, 'dend' + str(i), x = x, dx = dx, dia = args[4] )
            moose.connect( prev, 'axial', compt, 'raxial' )
            prev = compt
            x += dx
        self.elecid = cell
        return cell

    ################################################################
    def _buildVclampOnCompt( self, dendCompts, spineCompts, stimInfo ):
        # stimInfo = [path, geomExpr, relPath, field, expr_string]
        stimObj = []
        for i in dendCompts + spineCompts:
            vclamp = make_vclamp( name = 'vclamp', parent = i.path )

            # Assume SI units. Scale by Cm to get reasonable gain.
            vclamp.gain = i.Cm * 1e4 
            moose.connect( i, 'VmOut', vclamp, 'sensedIn' )
            moose.connect( vclamp, 'currentOut', i, 'injectMsg' )
            stimObj.append( vclamp )

        return stimObj

    def _buildSynInputOnCompt( self, dendCompts, spineCompts, stimInfo, doPeriodic = False ):
        # stimInfo = [path, geomExpr, relPath, field, expr_string]
        # Here we hack geomExpr to use it for the syn weight. We assume it
        # is just a number. In due course
        # it should be possible to actually evaluate it according to geom.
        synWeight = float( stimInfo.geom_expr )
        stimObj = []
        for i in dendCompts + spineCompts:
            path = i.path + '/' + stimInfo.relpath + '/sh/synapse[0]'
            if moose.exists( path ):
                synInput = make_synInput( name='synInput', parent=path )
                synInput.doPeriodic = doPeriodic
                moose.element(path).weight = synWeight
                moose.connect( synInput, 'spikeOut', path, 'addSpike' )
                stimObj.append( synInput )
        return stimObj
        
    ################################################################
    # Here we set up the distributions
    ################################################################
    def buildPassiveDistrib( self ):
	# [path field expr [field expr]...]
        # RM, RA, CM set specific values, per unit area etc.
        # Rm, Ra, Cm set absolute values.
        # Also does Em, Ek, initVm
	# Expression can use p, g, L, len, dia, maxP, maxG, maxL.
        temp = []
        for i in self.passiveDistrib:
            # Handle legacy format of ['.', path, field, expr [field expr]]
            if (len( i ) < 3) or (i[0] != '.' and len(i) %2 != 1):
                raise BuildError( "buildPassiveDistrib: Need 3 + N*2 arguments as (path field expr [field expr]...), have {}".format( len(i) ) )

            if not(( len(i) % 2 ) != 1 and i[0] == '.' ):
                temp.append( '.' )
            temp.extend( i )
            temp.extend( [""] )
        self.elecid.passiveDistribution = temp

    def buildChanDistrib( self ):
        temp = []
        for i in self.chanDistrib:
            temp.extend( i )
            temp.extend( [""] )
        self.elecid.channelDistribution = temp

    def buildSpineDistrib( self ):
        # For uniformity and conciseness, we don't use a dictionary.
        # ordering of spine distrib is
        # name, path, spacing, spacingDistrib, size, sizeDistrib, angle, angleDistrib
        # [i for i in L1 if i in L2]
        # The first two args are compulsory, and don't need arg keys.
        usageStr = 'Usage: name, path, [spacing, spacingDistrib, size, sizeDistrib, angle, angleDistrib]'
        temp = []
        defaults = ['spine', '#dend#,#apical#', '10e-6', '1e-6', '1', '0.5', '0', '6.2831853' ]
        argKeys = ['spacing', 'spacingDistrib', 'size', 'sizeDistrib', 'angle', 'angleDistrib' ]
        for i in self.spineDistrib:
            if len(i) >= 2 :
                arg = i[:2]
                # Backward compat hack here
                bcKeys = [ j for j in i[2:] if j in argKeys ]
                if len( bcKeys ) > 0: # Looks like we have an old arg str
                    print('Rdesigneur::buildSpineDistrib: Warning: Deprecated argument format.\nWill accept for now.')
                    print(usageStr)
                    temp.extend( i + [''] )
                elif len( i ) > len( defaults ):
                    print('Rdesigneur::buildSpineDistrib: Warning: too many arguments in spine definition')
                    print(usageStr)
                else:
                    optArg = i[2:] + defaults[ len(i):]
                    assert( len( optArg ) == len( argKeys ) )
                    for j in zip( argKeys, optArg ):
                        arg.extend( [j[0], j[1]] )
                    temp.extend( arg + [''] )

        self.elecid.spineDistribution = temp

    def buildChemDistrib( self ):
        # Orig format [chem, elecPath, install, expr]
        #   where chem and install were not being used.
        # Modified format [chemLibPath, elecPath, newChemName, expr]
        # chemLibPath is name of chemCompt or even group on library
        # If chemLibPath has multiple compts on it, then the smaller ones
        # become endoMeshes, scaled as per original.
        # As a backward compatibility hack, if the newChemName == 'install'
        # we use the default naming.
        for i in self.chemDistrib:
            pair = i[1] + " " + i[3]
            # Assign any other params. Possibly the first param should
            # be a global scaling factor.
            self.cellPortionElist = self.elecid.compartmentsFromExpression[ pair ]
            if len( self.cellPortionElist ) == 0:
                raise BuildError( \
                    "buildChemDistrib: No elec compartments found in path: '" \
                        + pair + "'" )
            self.spineComptElist = self.elecid.spinesFromExpression[ pair ]
            #print( 'LEN SPINECOMPTELIST =' + str( pair ) + ", " str( len( self.spineComptElist ) ) )
            '''
            if len( self.spineComptElist ) == 0:
                raise BuildError( \
                    "buildChemDistrib: No spine compartments found in path: '" \
                        + pair + "'" )
            '''
            # Build the neuroMesh
            # Check if it is good. Need to catch the ValueError here.
            self._buildNeuroMesh()
            # Assign the solvers

    ################################################################
    # Here we set up the adaptors
    ################################################################
    def findMeshOnName( self, name ):
        pos = name.find( '/' )
        if ( pos != -1 ):
            temp = name[:pos]
            if temp == 'psd' or temp == 'spine' or temp == 'dend':
                return ( temp, name[pos+1:] )
        return ("","")


    def buildAdaptors( self ):
        for i in self.adaptorList:
            mesh, name = self.findMeshOnName( i[0] )
            if mesh == "":
                mesh, name = self.findMeshOnName( i[2] )
                if  mesh == "":
                    raise BuildError( "buildAdaptors: Failed for " + i[2] )
                self._buildAdaptor( mesh, i[0], i[1], name, i[3], True, i[4], i[5] )
            else:
                self._buildAdaptor( mesh, i[2], i[3], name, i[1], False, i[4], i[5] )

    ################################################################
    # Here we set up the plots. Dummy for cases that don't match conditions
    ################################################################
    def _collapseElistToPathAndClass( self, comptList, path, className ):
        dummy = moose.element( '/' )
        ret = [ dummy ] * len( comptList )
        j = 0
        for i in comptList:
            if moose.exists( i.path + '/' + path ):
                obj = moose.element( i.path + '/' + path )
                if obj.isA[ className ]:
                    ret[j] = obj
            j += 1
        return ret

    # Utility function for doing lookups for objects.
    def _makeUniqueNameStr( self, obj ):
        # second one is faster than the former. 140 ns v/s 180 ns.
        #  return obj.name + " " + str( obj.index )
        return "%s %s" % (obj.name, obj.index)

    # Returns vector of source objects, and the field to use.
    # plotSpec is of the form
    #   [ region_wildcard, region_expr, path, field, title]
    def _parseComptField( self, comptList, plotSpec, knownFields ):
        # Put in stuff to go through fields if the target is a chem object
        field = plotSpec.field
        if not field in knownFields:
            print("Warning: Rdesigneur::_parseComptField: Unknown field '{}'".format( field ) )
            return (), ""

        kf = knownFields[field] # Find the field to decide type.
        if kf[0] in ['CaConcBase', 'ChanBase', 'NMDAChan', 'VClamp']:
            objList = self._collapseElistToPathAndClass( comptList, plotSpec.relpath, kf[0] )
            return objList, kf[1]
        elif field in [ 'n', 'conc', 'volume']:
            path = plotSpec.relpath
            pos = path.find( '/' )
            if pos == -1:   # Assume it is in the dend compartment.
                path  = 'dend/' + path
            pos = path.find( '/' )
            chemCompt = path[:pos]
            if chemCompt[-5:] == "_endo":
                chemCompt = chemCompt[0:-5]
            cc = moose.element(self.modelPath + '/chem/' + chemCompt)
            voxelVec = []
            temp = [ self._makeUniqueNameStr( i ) for i in comptList ]
            #print( temp )
            #print( "#####################" )
            comptSet = set( temp )
            #em = [ moose.element(i) for i in cc.elecComptMap ]
            em = sorted( [ self._makeUniqueNameStr(i[0]) for i in cc.elecComptMap ] )
            #print( em )
            #print( "=================================================" )

            voxelVec = [i for i in range(len( em ) ) if em[i] in comptSet ]
            # Here we collapse the voxelVec into objects to plot.
            allObj = moose.vec( self.modelPath + '/chem/' + plotSpec.relpath )
            #print "####### allObj=", self.modelPath + '/chem/' + plotSpec[2]
            if len( allObj ) >= len( voxelVec ):
                objList = [ allObj[int(j)] for j in voxelVec]
            else:
                objList = []
                print( "Warning: Rdesigneur::_parseComptField: unknown Object: '", plotSpec.relpath, "'" )
            #print "############", chemCompt, len(objList), kf[1]
            return objList, kf[1]

        else:
            return comptList, kf[1]


    def _buildPlots( self ):
        knownFields = {
            'Vm':('CompartmentBase', 'getVm', 1000, 'Memb. Potential (mV)' ),
            'spikeTime':('CompartmentBase', 'getVm', 1, 'Spike Times (s)'),
            'Im':('CompartmentBase', 'getIm', 1e9, 'Memb. current (nA)' ),
            'Cm':('CompartmentBase', 'getCm', 1e12, 'Memb. capacitance (pF)' ),
            'Rm':('CompartmentBase', 'getRm', 1e-9, 'Memb. Res (GOhm)' ),
            'Ra':('CompartmentBase', 'getRa', 1e-6, 'Axial Res (MOhm)' ),
            'inject':('CompartmentBase', 'getInject', 1e9, 'inject current (nA)' ),
            'Gbar':('ChanBase', 'getGbar', 1e9, 'chan max conductance (nS)' ),
            'modulation':('ChanBase', 'getModulation', 1, 'chan modulation (unitless)' ),
            'Gk':('ChanBase', 'getGk', 1e9, 'chan conductance (nS)' ),
            'Ik':('ChanBase', 'getIk', 1e9, 'chan current (nA)' ),
            'ICa':('NMDAChan', 'getICa', 1e9, 'Ca current (nA)' ),
            'Ca':('CaConcBase', 'getCa', 1e3, 'Ca conc (uM)' ),
            'n':('PoolBase', 'getN', 1, '# of molecules'),
            'conc':('PoolBase', 'getConc', 1000, 'Concentration (uM)' ),
            'volume':('PoolBase', 'getVolume', 1e18, 'Volume (um^3)' ),
            'current':('VClamp', 'getCurrent', 1e9, 'Holding Current (nA)')
        }
        graphs = moose.Neutral( self.modelPath + '/graphs' )
        dummy = moose.element( '/' )
        k = 0
        for i in self.plotList:
            pair = i.elecpath + ' ' + i.geom_expr
            dendCompts = self.elecid.compartmentsFromExpression[ pair ]
            spineCompts = self.elecid.spinesFromExpression[ pair ]
            plotObj, plotField = self._parseComptField( dendCompts, i, knownFields )
            plotObj2, plotField2 = self._parseComptField( spineCompts, i, knownFields )
            assert( plotField == plotField2 )
            plotObj3 = plotObj + plotObj2
            numPlots = sum( q != dummy for q in plotObj3 )
            #print( "PlotList: {0}: numobj={1}, field ={2}, nd={3}, ns={4}".format( pair, numPlots, plotField, len( dendCompts ), len( spineCompts ) ) )
            if numPlots > 0:
                tabname = graphs.path + '/plot' + str(k)
                scale = knownFields[i.field][2]
                units = knownFields[i.field][3]
                if i.mode == 'wave':
                    self.wavePlotNames.append( [ tabname, i.title, k, scale, units, i ] )
                else:
                    self.plotNames.append( [ tabname, i.title, k, scale, units, i.field, i.ymin, i.ymax ] )
                if len( i.saveFile ) > 4 and i.saveFile[-4] == '.xml' or i.saveFile:
                    self.saveNames.append( [ tabname, len(self.saveNames), scale, units, i ] )

                k += 1
                if i.field == 'n' or i.field == 'conc' or i.field == 'volume' or i.field == 'Gbar':
                    tabs = moose.Table2( tabname, numPlots )
                else:
                    tabs = moose.Table( tabname, numPlots )
                    if i.field == 'spikeTime':
                        tabs.vec.threshold = -0.02 # Threshold for classifying Vm as a spike.
                        tabs.vec.useSpikeMode = True # spike detect mode on

            vtabs = moose.vec( tabs )
            q = 0
            for p in [ x for x in plotObj3 if x != dummy ]:
                moose.connect( vtabs[q], 'requestOut', p, plotField )
                q += 1

    def _buildMoogli( self ):
        knownFields = {
            'Vm':('CompartmentBase', 'getVm', 1000, 'Memb. Potential (mV)', -80.0, 40.0 ),
            'initVm':('CompartmentBase', 'getInitVm', 1000, 'Init. Memb. Potl (mV)', -80.0, 40.0 ),
            'Im':('CompartmentBase', 'getIm', 1e9, 'Memb. current (nA)', -10.0, 10.0 ),
            'inject':('CompartmentBase', 'getInject', 1e9, 'inject current (nA)', -10.0, 10.0 ),
            'Gbar':('ChanBase', 'getGbar', 1e9, 'chan max conductance (nS)', 0.0, 1.0 ),
            'Gk':('ChanBase', 'getGk', 1e9, 'chan conductance (nS)', 0.0, 1.0 ),
            'Ik':('ChanBase', 'getIk', 1e9, 'chan current (nA)', -10.0, 10.0 ),
            'ICa':('NMDAChan', 'getICa', 1e9, 'Ca current (nA)', -10.0, 10.0 ),
            'Ca':('CaConcBase', 'getCa', 1e3, 'Ca conc (uM)', 0.0, 10.0 ),
            'n':('PoolBase', 'getN', 1, '# of molecules', 0.0, 200.0 ),
            'conc':('PoolBase', 'getConc', 1000, 'Concentration (uM)', 0.0, 2.0 ),
            'volume':('PoolBase', 'getVolume', 1e18, 'Volume (um^3)' )
        }
        moogliBase = moose.Neutral( self.modelPath + '/moogli' )
        k = 0
        for i in self.moogList:
            kf = knownFields[i.field]
            pair = i.elecpath + " " + i.geom_expr
            dendCompts = self.elecid.compartmentsFromExpression[ pair ]
            spineCompts = self.elecid.spinesFromExpression[ pair ]
            dendObj, mooField = self._parseComptField( dendCompts, i, knownFields )
            spineObj, mooField2 = self._parseComptField( spineCompts, i, knownFields )
            assert( mooField == mooField2 )
            mooObj3 = dendObj + spineObj
            numMoogli = len( mooObj3 )
            self.moogNames.append( rmoogli.makeMoogli( self, mooObj3, i, kf ) )


    ################################################################
    # Here we display the plots and moogli
    ################################################################

    def displayMoogli( self, moogliDt, runtime, rotation = math.pi/500.0, fullscreen = False, block = True, azim = 0.0, elev = 0.0 ):
        rmoogli.displayMoogli( self, moogliDt, runtime, rotation = rotation, fullscreen = fullscreen, azim = azim, elev = elev )
        pr = moose.PyRun( '/model/updateMoogli' )

        pr.runString = '''
import rdesigneur.rmoogli
rdesigneur.rmoogli.updateMoogliViewer()
'''
        moose.setClock( pr.tick, moogliDt )
        moose.reinit()
        moose.start( runtime )
        if block:
            self.display( len( self.moogNames ) + 1)

    def display( self, startIndex = 0, block=True ):
        for i in self.plotNames:
            plt.figure( i[2] + startIndex )
            plt.title( i[1] )
            plt.xlabel( "Time (s)" )
            plt.ylabel( i[4] )
            vtab = moose.vec( i[0] )
            if i[5] == 'spikeTime':
                k = 0
                tmax = moose.element( '/clock' ).currentTime
                for j in vtab: # Plot a raster
                    y = [k] * len( j.vector )
                    plt.plot( j.vector * i[3], y, linestyle = 'None', marker = '.', markersize = 10 )
                    plt.xlim( 0, tmax )
                
            else:
                t = np.arange( 0, vtab[0].vector.size, 1 ) * vtab[0].dt
                for j in vtab:
                    plt.plot( t, j.vector * i[3] )
        if len( self.moogList ) or len( self.wavePlotNames ) > 0:
            plt.ion()
        # Here we build the plots and lines for the waveplots
        self.initWavePlots( startIndex )
        if len( self.wavePlotNames ) > 0:
            for i in range( 3 ):
                self.displayWavePlots()
        plt.show( block=block )
        self._save()                                            
        

    def initWavePlots( self, startIndex ):
        self.frameDt = moose.element( '/clock' ).currentTime/self.numWaveFrames
        for wpn in range( len(self.wavePlotNames) ):
            i = self.wavePlotNames[wpn]
            vtab = moose.vec( i[0] )
            if len( vtab ) < 2:
                print( "Warning: Waveplot {} abandoned, only {} points".format( i[1], len( vtab ) ) )
                continue
            dFrame = len( vtab[0].vector ) // self.numWaveFrames
            if dFrame < 1:
                dFrame = 1
            vpts = np.array( [ [k.vector[j] for j in range( 0, len( k.vector ), dFrame ) ] for k in vtab] ).T * i[3]
            fig = plt.figure( i[2] + startIndex )
            ax = fig.add_subplot( 111 )
            plt.title( i[1] )
            plt.xlabel( "position (voxels)" )
            plt.ylabel( i[4] )
            plotinfo = i[5]
            if plotinfo.ymin == plotinfo.ymax:
                mn = np.min(vpts)
                mx = np.max(vpts)
                if mn/mx < 0.3:
                    mn = 0
            else:
                mn = plotinfo.ymin
                mx = plotinfo.ymax
            ax.set_ylim( mn, mx )
            line, = plt.plot( range( len( vtab ) ), vpts[0] )
            timeLabel = plt.text( len(vtab ) * 0.05, mn + 0.9*(mx-mn), 'time = 0' )
            self.wavePlotNames[wpn].append( [fig, line, vpts, timeLabel] )
            fig.canvas.draw()

    def displayWavePlots( self ):
        for f in range( self.numWaveFrames ):
            for i in self.wavePlotNames:
                wp = i[6]
                if len( wp[2] ) > f:
                    wp[1].set_ydata( wp[2][f] )
                    wp[3].set_text( "time = {:.1f}".format(f*self.frameDt) )
                    wp[0].canvas.draw()
            #plt.pause(0.001)
        
        #This calls the _save function which saves only if the filenames have been specified

    ################################################################
    # Here we get the time-series data and write to various formats
    ################################################################
    #[TO DO] Add NSDF output function
    '''
    The original author of the functions -- [_savePlots(), _writeXML(), _writeCSV(), _save()] is
    Sarthak Sharma.
    Email address: sarthaks442@gmail.com
    Heavily modified by U.S. Bhalla
    '''
    def _writeXML( self, plotData, time, vtab ): 
        tabname = plotData[0]
        idx = plotData[1]
        scale = plotData[2]
        units = plotData[3]
        rp = plotData[4]
        filename = rp.saveFile[:-4] + str(idx) + '.xml'
        root = etree.Element("TimeSeriesPlot")
        parameters = etree.SubElement( root, "parameters" )
        if self.params == None:
            parameters.text = "None"
        else:
            assert(isinstance(self.params, dict)), "'params' should be a dictionary."
            for pkey, pvalue in self.params.items():
                parameter = etree.SubElement( parameters, str(pkey) )
                parameter.text = str(pvalue)

        #plotData contains all the details of a single plot
        title = etree.SubElement( root, "timeSeries" )
        title.set( 'title', rp.title)
        title.set( 'field', rp.field)
        title.set( 'scale', str(scale) )
        title.set( 'units', units)
        title.set( 'dt', str(vtab[0].dt) )
        res = rp.saveResolution
        p = []
        for t, v in zip( time, vtab ):
            p.append( etree.SubElement( title, "data"))
            p[-1].set( 'path', v.path )
            p[-1].text = ''.join( str(round(y,res)) + ' ' for y in v.vector )
        tree = etree.ElementTree(root)
        tree.write(filename)

    def _writeCSV( self, plotData, time, vtab ): 
        tabname = plotData[0]
        idx = plotData[1]
        scale = plotData[2]
        units = plotData[3]
        rp = plotData[4]
        filename = rp.saveFile[:-4] + str(idx) + '.csv'

        header = ["time",]
        valMatrix = [time,]
        header.extend( [ v.path for v in vtab ] )
        valMatrix.extend( [ v.vector for v in vtab ] )
        nv = np.array( valMatrix ).T
        with open(filename, 'wb') as f:
            writer = csv.writer(f, quoting=csv.QUOTE_MINIMAL)
            writer.writerow(header)
            for row in nv:
                writer.writerow(row)

    ##########****SAVING*****###############


    def _save( self ):
        for i in self.saveNames:
            tabname = i[0]
            idx = i[1]
            scale = i[2]
            units = i[3]
            rp = i[4] # The rplot data structure, it has the setup info.

            vtab = moose.vec( tabname )
            t = np.arange( 0, vtab[0].vector.size, 1 ) * vtab[0].dt
            ftype = rp.filename[-4:]
            if ftype == '.xml':
                self._writeXML( i, t, vtab )
            elif ftype == '.csv':
                self._writeCSV( i, t, vtab )
            else:
                print("Save format '{}' not known, please use .csv or .xml".format( ftype ) )

    ################################################################
    # Here we set up the stims
    ################################################################
    def _buildStims( self ):
        knownFields = {
                'inject':('CompartmentBase', 'setInject'),
                'Ca':('CaConcBase', 'setCa'),
                'n':('PoolBase', 'setN'),
                'conc':('PoolBase', 'setConc'),
                'nInit':('PoolBase', 'setNinit'),
                'concInit':('PoolBase', 'setConcInit'),
                'vclamp':('CompartmentBase', 'setInject'),
                'randsyn':('SynChan', 'addSpike'),
                'periodicsyn':('SynChan', 'addSpike')
        }
        stims = moose.Neutral( self.modelPath + '/stims' )
        k = 0
        # rstim class has {elecpath, geom_expr, relpath, field, expr}
        for i in self.stimList:
            pair = i.elecpath + " " + i.geom_expr
            dendCompts = self.elecid.compartmentsFromExpression[ pair ]
            spineCompts = self.elecid.spinesFromExpression[ pair ]
            #print( "pair = {}, numcompts = {},{} ".format( pair, len( dendCompts), len( spineCompts ) ) )
            if i.field == 'vclamp':
                stimObj3 = self._buildVclampOnCompt( dendCompts, spineCompts, i )
                stimField = 'commandIn'
            elif i.field == 'randsyn':
                stimObj3 = self._buildSynInputOnCompt( dendCompts, spineCompts, i )
                stimField = 'setRate'
            elif i.field == 'periodicsyn':
                stimObj3 = self._buildSynInputOnCompt( dendCompts, spineCompts, i, doPeriodic = True )
                stimField = 'setRate'
            else:
                stimObj, stimField = self._parseComptField( dendCompts, i, knownFields )
                stimObj2, stimField2 = self._parseComptField( spineCompts, i, knownFields )
                assert( stimField == stimField2 )
                stimObj3 = stimObj + stimObj2
            numStim = len( stimObj3 )
            if numStim > 0:
                funcname = stims.path + '/stim' + str(k)
                k += 1
                func = moose.Function( funcname )
                func.expr = i.expr
                #if i[3] == 'vclamp': # Hack to clean up initial condition
                func.doEvalAtReinit = 1
                for q in stimObj3:
                    moose.connect( func, 'valueOut', q, stimField )

    ################################################################
    def _configureHSolve( self ):
        if not self.turnOffElec:
            hsolve = moose.HSolve( self.elecid.path + '/hsolve' )
            hsolve.dt = self.elecDt
            hsolve.target = self.soma.path

# Utility function for setting up clocks.
    def _configureClocks( self ):
        if self.turnOffElec:
            elecDt = 1e6
            elecPlotDt = 1e6
        else:
            elecDt = self.elecDt
            elecPlotDt = self.elecPlotDt
        diffDt = self.diffDt
        chemDt = self.chemDt
        for i in range( 0, 9 ):     # Assign elec family of clocks
            moose.setClock( i, elecDt )
        moose.setClock( 8, elecPlotDt ) 
        moose.setClock( 10, diffDt )# Assign diffusion clock.
        for i in range( 11, 18 ):   # Assign the chem family of clocks.
            moose.setClock( i, chemDt )
        if not self.turnOffElec:    # Assign the Function clock
            moose.setClock( 12, self.funcDt )
        moose.setClock( 18, self.chemPlotDt )
        sys.stdout.flush()
    ################################################################
    ################################################################
    ################################################################

    def validateFromMemory( self, epath, cpath ):
        return self.validateChem()

    #################################################################
    # assumes ePath is the parent element of the electrical model,
    # and cPath the parent element of the compts in the chem model
    def buildFromMemory( self, ePath, cPath, doCopy = False ):
        if not self.validateFromMemory( ePath, cPath ):
            return
        if doCopy:
            x = moose.copy( cPath, self.model )
            self.chemid = moose.element( x )
            self.chemid.name = 'chem'
            x = moose.copy( ePath, self.model )
            self.elecid = moose.element( x )
            self.elecid.name = 'elec'
        else:
            self.elecid = moose.element( ePath )
            self.chemid = moose.element( cPath )
            if self.elecid.path != self.model.path + '/elec':
                if ( self.elecid.parent != self.model ):
                    moose.move( self.elecid, self.model )
                self.elecid.name = 'elec'
            if self.chemid.path != self.model.path + '/chem':
                if ( self.chemid.parent != self.model ):
                    moose.move( self.chemid, self.model )
                self.chemid.name = 'chem'


        ep = self.elecid.path
        somaList = moose.wildcardFind( ep + '/#oma#[ISA=CompartmentBase]' )
        if len( somaList ) == 0:
            somaList = moose.wildcardFind( ep + '/#[ISA=CompartmentBase]' )
        assert( len( somaList ) > 0 )
        maxdia = 0.0
        for i in somaList:
            if ( i.diameter > maxdia ):
                self.soma = i
        #self.soma = self.comptList[0]
        self._decorateWithSpines()
        self.spineList = moose.wildcardFind( ep + '/#spine#[ISA=CompartmentBase],' + ep + '/#head#[ISA=CompartmentBase]' )
        if len( self.spineList ) == 0:
            self.spineList = moose.wildcardFind( ep + '/#head#[ISA=CompartmentBase]' )
        nmdarList = moose.wildcardFind( ep + '/##[ISA=NMDAChan]' )

        self.comptList = moose.wildcardFind( ep + '/#[ISA=CompartmentBase]')
        print("Rdesigneur: Elec model has ", len( self.comptList ),
            " compartments and ", len( self.spineList ),
            " spines with ", len( nmdarList ), " NMDARs")


        self._buildNeuroMesh()

        self._configureSolvers()
        for i in self.adaptorList:
            #  print(i)
            assert len(i) >= 8
            self._buildAdaptor( i[0],i[1],i[2],i[3],i[4],i[5],i[6],i[7] )

    ################################################################

    def buildFromFile( self, efile, cfile ):
        self.efile = efile
        self.cfile = cfile
        self._loadElec( efile, 'tempelec' )
        if len( self.chanDistrib ) > 0:
            self.elecid.channelDistribution = self.chanDistrib
            self.elecid.parseChanDistrib()
        self._loadChem( cfile, 'tempchem' )
        self.buildFromMemory( self.model.path + '/tempelec', self.model.path + '/tempchem' )

    ################################################################
    # Utility function to add a single spine to the given parent.

    # parent is parent compartment for this spine.
    # spineProto is just that.
    # pos is position (in metres ) along parent compartment
    # angle is angle (in radians) to rotate spine wrt x in plane xy.
    # Size is size scaling factor, 1 leaves as is.
    # x, y, z are unit vectors. Z is along the parent compt.
    # We first shift the spine over so that it is offset by the parent compt
    # diameter.
    # We then need to reorient the spine which lies along (i,0,0) to
    #   lie along x. X is a unit vector so this is done simply by
    #   multiplying each coord of the spine by x.
    # Finally we rotate the spine around the z axis by the specified angle
    # k is index of this spine.
    def _addSpine( self, parent, spineProto, pos, angle, x, y, z, size, k ):
        spine = moose.copy( spineProto, parent.parent, 'spine' + str(k) )
        kids = spine[0].children
        coords = []
        ppos = np.array( [parent.x0, parent.y0, parent.z0] )
        for i in kids:
            #print i.name, k
            j = i[0]
            j.name += str(k)
            #print 'j = ', j
            coords.append( [j.x0, j.y0, j.z0] )
            coords.append( [j.x, j.y, j.z] )
            self._scaleSpineCompt( j, size )
            moose.move( i, self.elecid )
        origin = coords[0]
        #print 'coords = ', coords
        # Offset it so shaft starts from surface of parent cylinder
        origin[0] -= parent.diameter / 2.0
        coords = np.array( coords )
        coords -= origin # place spine shaft base at origin.
        rot = np.array( [x, [0,0,0], [0,0,0]] )
        coords = np.dot( coords, rot )
        moose.delete( spine )
        moose.connect( parent, "raxial", kids[0], "axial" )
        self._reorientSpine( kids, coords, ppos, pos, size, angle, x, y, z )

    ################################################################
    ## The spineid is the parent object of the prototype spine. The
    ## spine prototype can include any number of compartments, and each
    ## can have any number of voltage and ligand-gated channels, as well
    ## as CaConc and other mechanisms.
    ## The parentList is a list of Object Ids for parent compartments for
    ## the new spines
    ## The spacingDistrib is the width of a normal distribution around
    ## the spacing. Both are in metre units.
    ## The reference angle of 0 radians is facing away from the soma.
    ## In all cases we assume that the spine will be rotated so that its
    ## axis is perpendicular to the axis of the dendrite.
    ## The simplest way to put the spine in any random position is to have
    ## an angleDistrib of 2 pi. The algorithm selects any angle in the
    ## linear range of the angle distrib to add to the specified angle.
    ## With each position along the dendrite the algorithm computes a new
    ## spine direction, using rotation to increment the angle.
    ################################################################
    def _decorateWithSpines( self ):
        args = []
        for i in self.addSpineList:
            if not moose.exists( '/library/' + i[0] ):
                print('Warning: _decorateWithSpines: spine proto ', i[0], ' not found.')
                continue
            s = ""
            for j in range( 9 ):
                s = s + str(i[j]) + ' '
            args.append( s )
        self.elecid.spineSpecification = args
        self.elecid.parseSpines()

    ################################################################

    def _loadElec( self, efile, elecname ):
        if ( efile[ len( efile ) - 2:] == ".p" ):
            self.elecid = moose.loadModel( efile, '/library/' + elecname)
            print(self.elecid)
        elif ( efile[ len( efile ) - 4:] == ".swc" ):
            self.elecid = moose.loadModel( efile, '/library/' + elecname)
        else:
            nm = NeuroML()
            print("in _loadElec, combineSegments = ", self.combineSegments)
            nm.readNeuroMLFromFile( efile, \
                    params = {'combineSegments': self.combineSegments, \
                    'createPotentialSynapses': True } )
            if moose.exists( '/cells' ):
                kids = moose.wildcardFind( '/cells/#' )
            else:
                kids = moose.wildcardFind( '/library/#[ISA=Neuron],/library/#[TYPE=Neutral]' )
                if ( kids[0].name == 'spine' ):
                    kids = kids[1:]

            assert( len( kids ) > 0 )
            self.elecid = kids[0]
            temp = moose.wildcardFind( self.elecid.path + '/#[ISA=CompartmentBase]' )

        transformNMDAR( self.elecid.path )
        kids = moose.wildcardFind( '/library/##[0]' )
        for i in kids:
            i.tick = -1


    #################################################################

    # This assumes that the chemid is located in self.parent.path+/chem
    # It moves the existing chem compartments into a NeuroMesh
    # For now this requires that we have a dend, a spine and a PSD,
    # with those names and volumes in decreasing order.
    def validateChem( self  ):
        cpath = self.chemid.path
        comptlist = moose.wildcardFind( cpath + '/#[ISA=ChemCompt]' )
        if len( comptlist ) == 0:
            raise BuildError( "validateChem: no compartment on: " + cpath )

        return True

        '''
        if len( comptlist ) == 1:
            return;

        # Sort comptlist in decreasing order of volume
        sortedComptlist = sorted( comptlist, key=lambda x: -x.volume )
        if ( len( sortedComptlist ) != 3 ):
            print(cpath, sortedComptlist)
            raise BuildError( "validateChem: Require 3 chem compartments, have: " + str( len( sortedComptlist ) ) )
        '''
        '''
        if not( sortedComptlist[0].name.lower() == 'dend' and \
            sortedComptlist[1].name.lower() == 'spine' and \
            sortedComptlist[2].name.lower() == 'psd' ):
            raise BuildError( "validateChem: Invalid compt names: require dend, spine and PSD.\nActual names = " \
                    + sortedComptList[0].name + ", " \
                    + sortedComptList[1].name + ", " \
                    + sortedComptList[2].name )
        '''

    #################################################################

    def _isModelFromKkit( self ):
        for i in self.chemProtoList:
            if i[0][-2:] == ".g":
                return True
        return False

    def _assignComptNamesFromKkit( self ):
        '''
        Algorithm: Identify compts by volume. Assume a couple of standard
        orders depending on the addSomaChemCompt and addEndoChemCompt
        flags:\n
        ascc = 0, aecc = 0: dend, spine, psd.\n
        ascc = 0, aecc = 1: dend, dend_endo, spine, spine_endo, psd, psd_endo.\n
        ascc = 1, aecc = 0: soma, dend, spine, psd.\n
        ascc = 1, aecc = 1: soma, soma_endo, dend, dend_endo, spine, spine_endo, psd, psd_endo.\n
        In all cases, a shorter list of chem compartments will only fill
        up the list to the available length.\n
        soma_endo can be thought of as nucleus.\n
        psd_endo doesn't really make sense, as peri-synaptic region really
        needs to talk both to PSD and to spine bulk.
        '''
        comptList = moose.wildcardFind( self.chemid.path + '/#[ISA=ChemCompt]' )
        #print( "comptList = {}".format ( [i.name for i in comptList]) )
        if len( comptList ) < 2:
            if comptList[0].name != 'dend':
                comptList[0].name = 'dend'
            return comptList
        if not self._isModelFromKkit():
            return comptList
        sortedComptList = sorted( comptList, key=lambda x: -x.volume )
        if self.addSomaChemCompt:
            if self.addEndoChemCompt:
                sortedNames = ["soma", "soma_endo", "dend", "dend_endo", "spine", "spine_endo", "psd", "psd_endo", ]
            else:
                sortedNames = ["soma", "dend", "spine","psd"]
        else:
            if self.addEndoChemCompt:
                sortedNames = ["dend", "dend_endo", "spine", "spine_endo", "psd", "psd_endo", ]
            else:
                sortedNames = ["dend", "spine","psd"]

        #print( "sortedNames = {}".format( sortedNames ) )
        for i in range(min( len( sortedComptList ), len( sortedNames ) ) ):
            #print( "SortedClist= {}".format( sortedComptList[i] ))
            if sortedComptList[i].name != sortedNames[i]:
                sortedComptList[i].name = sortedNames[i]
            #print( "SortedClist= {}     {}".format( sortedNames[i], sortedComptList[i].volume ))
        return sortedComptList


    def _buildNeuroMesh( self ):
        # Address the following cases:
        #   - dend alone
        #   - dend with spine and PSD
        #   - above plus n endo meshes located as per name suffix: 
        #       er_1_dend, vesicle_2_spine, 
        #   - above plus presyn, e.g.:
        #       er_1_dend, vesicle_2_spine, 
        #   - above plus soma
        #   - soma alone
        #   - soma plus n endo meshes

        self.chemid.name = 'temp_chem'
        newChemid = moose.Neutral( self.model.path + '/chem' )
        comptlist = self._assignComptNamesFromKkit()
        comptdict = { i.name:i for i in comptlist }
        if len(comptdict) == 1 or 'dend' in comptdict:
            self.dendCompt = moose.NeuroMesh( newChemid.path + '/dend' )
            self.dendCompt.geometryPolicy = 'cylinder'
            self.dendCompt.separateSpines = 0
            self._moveCompt( comptdict['dend'], self.dendCompt )
            comptdict['dend'] = self.dendCompt

        if 'dend' in comptdict and 'spine' in comptdict:
            #print( "comptdict = {}".format (comptdict ) )
            self.dendCompt.separateSpines = 1
            self.spineCompt = moose.SpineMesh( newChemid.path + '/spine' )
            moose.connect( self.dendCompt, 'spineListOut', self.spineCompt, 'spineList' )
            self._moveCompt( comptdict['spine'], self.spineCompt )
            comptdict['spine'] = self.spineCompt
            # We need to make a PSD in the spine even if it is uninhabited.
            self.psdCompt = moose.PsdMesh( newChemid.path + '/psd' )
            moose.connect( self.dendCompt, 'psdListOut', self.psdCompt, 'psdList','OneToOne')
            if 'psd' in comptdict: # Shift stuff over if any.
                self._moveCompt( comptdict['psd'], self.psdCompt )
                comptdict['psd'] = self.psdCompt

        self.dendCompt.diffLength = self.diffusionLength
        self.dendCompt.subTree = self.cellPortionElist
        for i in comptdict:
            if len(i) > 5:
                if i[-5:] == '_endo':
                    endo = moose.EndoMesh( newChemid.path + '/' +i )
                    surround = comptdict[i[0:-5]]
                    self._endos.append( [endo, surround] )
                    #print( "{}****{}".format(i[0:-5], comptdict[i[0:-5]]))
                    self._moveCompt( comptdict[i], endo )
                    comptdict[i] = endo
        moose.delete( self.chemid )
        self.chemid = newChemid


    #################################################################
    def _configureSolvers( self ) :
        if not hasattr( self, 'chemid' ) or len( self.chemDistrib ) == 0:
            return
        if not hasattr( self, 'dendCompt' ):
            raise BuildError( "configureSolvers: no chem meshes defined." )
        fixXreacs.fixXreacs( self.chemid.path )
        dmksolve = moose.Ksolve( self.dendCompt.path + '/ksolve' )
        dmksolve.method = self.ode_method
        dmdsolve = moose.Dsolve( self.dendCompt.path + '/dsolve' )
        dmstoich = moose.Stoich( self.dendCompt.path + '/stoich' )
        dmstoich.compartment = self.dendCompt
        dmstoich.ksolve = dmksolve
        dmstoich.dsolve = dmdsolve
        dmstoich.reacSystemPath = self.dendCompt.path + "/##"
        # Below we have code that only applies if there are spines
        # Put in spine solvers. Note that these get info from the dendCompt
        if hasattr( self, 'spineCompt' ):
            if self.useGssa:
                smksolve = moose.Gsolve( self.spineCompt.path + '/ksolve' )
            else:
                smksolve = moose.Ksolve( self.spineCompt.path + '/ksolve' )
                smksolve.method = self.ode_method
            smdsolve = moose.Dsolve( self.spineCompt.path + '/dsolve' )
            smstoich = moose.Stoich( self.spineCompt.path + '/stoich' )
            smstoich.compartment = self.spineCompt
            smstoich.ksolve = smksolve
            smstoich.dsolve = smdsolve
            smstoich.reacSystemPath = self.spineCompt.path + "/##"
            # Put in PSD solvers. Note that these get info from the dendCompt
            if self.useGssa:
                pmksolve = moose.Gsolve( self.psdCompt.path + '/ksolve' )
            else:
                pmksolve = moose.Ksolve( self.psdCompt.path + '/ksolve' )
                pmksolve.method = self.ode_method
            pmdsolve = moose.Dsolve( self.psdCompt.path + '/dsolve' )
            pmstoich = moose.Stoich( self.psdCompt.path + '/stoich' )
            pmstoich.compartment = self.psdCompt
            pmstoich.ksolve = pmksolve
            pmstoich.dsolve = pmdsolve
            if len( moose.wildcardFind( 'self.psdCompt.path/##[ISA=PoolBase]' ) ) == 0:
                moose.Pool( self.psdCompt.path + '/dummy' )
            pmstoich.reacSystemPath = self.psdCompt.path + "/##"

            # Here we should test what kind of geom we have to use
            # Put in cross-compartment diffusion between ksolvers
            dmdsolve.buildNeuroMeshJunctions( smdsolve, pmdsolve )
            # All the Xreac stuff in the solvers is now eliminated.

            # set up the connections so that the spine volume scaling can happen
            self.elecid.setSpineAndPsdMesh( self.spineCompt, self.psdCompt)
            self.elecid.setSpineAndPsdDsolve( smdsolve, pmdsolve )
        for i in self._endos:
            i[0].isMembraneBound = True
            i[0].surround = i[1]
            #i[0].elecComptMap = i[1].elecComptMap
            path = i[0].path
            #print( "Doing endo {} inside {}".format( path, i[1].path ) )
            if self.useGssa:
                eksolve = moose.Gsolve( path + '/ksolve' )
            else:
                eksolve = moose.Ksolve( path + '/ksolve' )
            edsolve = moose.Dsolve( path + '/dsolve' )
            estoich = moose.Stoich( path + '/stoich' )
            estoich.compartment = i[0]
            estoich.ksolve = eksolve
            estoich.dsolve = edsolve
            estoich.reacSystemPath = path + "/##"
            edsolve.buildMeshJunctions( moose.element(i[1].path + '/dsolve' ))
    ################################################################

    def _loadChem( self, fname, chemName ):
        chem = moose.Neutral( '/library/' + chemName )
        pre, ext = os.path.splitext( fname )
        if ext == '.xml' or ext == '.sbml':
            modelId = moose.mooseReadSBML( fname, chem.path )
        else:
            modelId = moose.loadModel( fname, chem.path, 'ee' )
        comptlist = moose.wildcardFind( chem.path + '/#[ISA=ChemCompt]' )
        if len( comptlist ) == 0:
            print("loadChem: No compartment found in file: ", fname)
            return
        # Sort comptlist in decreasing order of volume
        '''
        sortedComptlist = sorted( comptlist, key=lambda x: -x.volume )
        if ( len( sortedComptlist ) >= 1 ):
            sortedComptlist[0].name = 'dend'
        if ( len( sortedComptlist ) >= 2 ):
            sortedComptlist[1].name = 'spine'
        if ( len( sortedComptlist ) >= 3 ):
            sortedComptlist[2].name = 'psd'
        '''

    ################################################################

    def _moveCompt( self, a, b ):
        b.setVolumeNotRates( a.volume )
        for i in moose.wildcardFind( a.path + '/#' ):
            if ( i.name != 'mesh' ):
                moose.move( i, b )
                #print( "Moving {} {} to {}".format( i.className, i.name, b.name ))
        moose.delete( a )
    ################################################################
    def _buildAdaptor( self, meshName, elecRelPath, elecField, \
            chemRelPath, chemField, isElecToChem, offset, scale ):
        #print "offset = ", offset, ", scale = ", scale
        mesh = moose.element( '/model/chem/' + meshName )
        #elecComptList = mesh.elecComptList
        if elecRelPath == 'spine':
            # This is nasty. The spine indexing is different from
            # the compartment indexing and the mesh indexing and the 
            # chem indexing. Need to fix at some time.
            #elecComptList = moose.vec( mesh.elecComptList[0].path + '/../spine' )
            elecComptList = moose.element( '/model/elec').spineIdsFromCompartmentIds[ mesh.elecComptList ]
            #print( len( mesh.elecComptList ) )
            # for i,j in zip( elecComptList, mesh.elecComptList ):
            #    print( "Lookup: {} {} {}; orig: {} {} {}".format( i.name, i.index, i.fieldIndex, j.name, j.index, j.fieldIndex ))
        else:
            elecComptList = mesh.elecComptList

        if len( elecComptList ) == 0:
            raise BuildError( \
                "buildAdaptor: no elec compts in elecComptList on: " + \
                mesh.path )
        startVoxelInCompt = mesh.startVoxelInCompt
        endVoxelInCompt = mesh.endVoxelInCompt
        capField = elecField[0].capitalize() + elecField[1:]
        capChemField = chemField[0].capitalize() + chemField[1:]
        chemPath = mesh.path + '/' + chemRelPath
        if not( moose.exists( chemPath ) ):
            raise BuildError( \
                "Error: buildAdaptor: no chem obj in " + chemPath )
        chemObj = moose.element( chemPath )
        assert( chemObj.numData >= len( elecComptList ) )
        adName = '/adapt'
        for i in range( 1, len( elecRelPath ) ):
            if ( elecRelPath[-i] == '/' ):
                adName += elecRelPath[1-i]
                break
        ad = moose.Adaptor( chemObj.path + adName, len( elecComptList ) )
        #print 'building ', len( elecComptList ), 'adaptors ', adName, ' for: ', mesh.name, elecRelPath, elecField, chemRelPath
        av = ad.vec
        chemVec = moose.element( mesh.path + '/' + chemRelPath ).vec
        root = moose.element( '/' )

        for i in zip( elecComptList, startVoxelInCompt, endVoxelInCompt, av ):
            i[3].inputOffset = 0.0
            i[3].outputOffset = offset
            i[3].scale = scale
            if elecRelPath == 'spine':
                # Check needed in case there were unmapped entries in 
                # spineIdsFromCompartmentIds
                elObj = i[0]
                if elObj == root:
                    continue
            else:
                ePath = i[0].path + '/' + elecRelPath
                if not( moose.exists( ePath ) ):
                    continue
                    #raise BuildError( "Error: buildAdaptor: no elec obj in " + ePath )
                elObj = moose.element( i[0].path + '/' + elecRelPath )
            if ( isElecToChem ):
                elecFieldSrc = 'get' + capField
                chemFieldDest = 'set' + capChemField
                #print ePath, elecFieldSrc, scale
                moose.connect( i[3], 'requestOut', elObj, elecFieldSrc )
                for j in range( i[1], i[2] ):
                    moose.connect( i[3], 'output', chemVec[j],chemFieldDest)
            else:
                chemFieldSrc = 'get' + capChemField
                elecFieldDest = 'set' + capField
                for j in range( i[1], i[2] ):
                    moose.connect( i[3], 'requestOut', chemVec[j], chemFieldSrc)
                msg = moose.connect( i[3], 'output', elObj, elecFieldDest )


#######################################################################
# Some helper classes, used to define argument lists.
#######################################################################

class baseplot:
    def __init__( self,
            elecpath='soma', geom_expr='1', relpath='.', field='Vm' ):
        self.elecpath = elecpath
        self.geom_expr = geom_expr
        self.relpath = relpath
        self.field = field

class rplot( baseplot ):
    def __init__( self,
        elecpath = 'soma', geom_expr = '1', relpath = '.', field = 'Vm', 
        title = 'Membrane potential', 
        mode = 'time', 
        ymin = 0.0, ymax = 0.0, 
        saveFile = "", saveResolution = 3, show = True ):
        baseplot.__init__( self, elecpath, geom_expr, relpath, field )
        self.title = title
        self.mode = mode # Options: time, wave, wave_still, raster
        self.ymin = ymin # If ymin == ymax, it autoscales.
        self.ymax = ymax
        if len( saveFile ) < 5:
            self.saveFile = ""
        else:
            f = saveFile.split('.')
            if len(f) < 2 or ( f[-1] != 'xml' and f[-1] != 'csv' ):
                raise BuildError( "rplot: Filetype is '{}', must be of type .xml or .csv.".format( f[-1] ) )
        self.saveFile = saveFile
        self.show = show

    def printme( self ):
        print( "{}, {}, {}, {}, {}, {}, {}, {}, {}, {}".format( 
            self.elecpath,
            self.geom_expr, self.relpath, self.field, self.title,
            self.mode, self.ymin, self.ymax, self.saveFile, self.show ) )

    @staticmethod
    def convertArg( arg ):
        if isinstance( arg, rplot ):
            return arg
        elif isinstance( arg, list ):
            return rplot( *arg )
        else:
            raise BuildError( "rplot initialization failed" )

class rmoog( baseplot ):
    def __init__( self,
        elecpath = 'soma', geom_expr = '1', relpath = '.', field = 'Vm', 
        title = 'Membrane potential', 
        ymin = 0.0, ymax = 0.0, 
        show = True ): # Could put in other display options.
        baseplot.__init__( self, elecpath, geom_expr, relpath, field )
        self.title = title
        self.ymin = ymin # If ymin == ymax, it autoscales.
        self.ymax = ymax
        self.show = show

    @staticmethod
    def convertArg( arg ):
        if isinstance( arg, rmoog ):
            return arg
        elif isinstance( arg, list ):
            return rmoog( *arg )
        else:
            raise BuildError( "rmoog initialization failed" )

class rstim( baseplot ):
    def __init__( self,
            elecpath = 'soma', geom_expr = '1', relpath = '.', field = 'inject', expr = '0'):
        baseplot.__init__( self, elecpath, geom_expr, relpath, field )
        self.expr = expr

    def printme( self ):
        print( "{0}, {1}, {2}, {3}, {4}".format( 
            self.elecpath, self.geom_expr, self.relpath, self.field, self.expr
            ) )

    @staticmethod
    def convertArg( arg ):
        if isinstance( arg, rstim ):
            return arg
        elif isinstance( arg, list ):
            return rstim( *arg )
        else:
            raise BuildError( "rstim initialization failed" )




"""test_moose_attribs.py: 

"""

__author__ = "Dilawar Singh"
__copyright__ = "Copyright 2017-, Dilawar Singh"
__version__ = "1.0.0"
__maintainer__ = "Dilawar Singh"
__email__ = "dilawars@ncbs.res.in"
__status__ = "Development"

import sys
import os
import moose

print(moose.about())

attribs = [
    "AdExIF",
    "AdThreshIF",
    "Adaptor",
    "Annotator",
    "Arith",
    "BufPool",
    "CaConc",
    "CaConcBase",
    "ChanBase",
    "ChemCompt",
    "Cinfo",
    "Clock",
    "Compartment",
    "CompartmentBase",
    "ConcChan",
    "CplxEnzBase",
    "CubeMesh",
    "CylMesh",
    "DiagonalMsg",
    "DifBuffer",
    "DifBufferBase",
    "DifShell",
    "DifShellBase",
    "DiffAmp",
    "Dsolve",
    "EndoMesh",
    "Enz",
    "EnzBase",
    "ExIF",
    "Finfo",
    "Function",
    "GapJunction",
    "GraupnerBrunel2012CaPlasticitySynHandler",
    "Group",
    "Gsolve",
    "HHChannel",
    "HHChannel2D",
    "HHChannelBase",
    "HHGate",
    "HHGate2D",
    "HSolve",
    "IntFire",
    "IntFireBase",
    "Interpol",
    "Interpol2D",
    "IzhIF",
    "IzhikevichNrn",
    "Ksolve",
    "LIF",
    "Leakage",
    "MMPump",
    "MMenz",
    "MarkovChannel",
    "MarkovOdeSolver",
    "MarkovRateTable",
    "MarkovSolver",
    "MarkovSolverBase",
    "MeshEntry",
    "MgBlock",
    "Msg",
    "Mstring",
    "NMDAChan",
    "Nernst",
    "NeuroMesh",
    "Neuron",
    "Neutral",
    "OneToAllMsg",
    "OneToOneDataIndexMsg",
    "OneToOneMsg",
    "PIDController",
    "Pool",
    "PoolBase",
    "PostMaster",
    "PsdMesh",
    "PulseGen",
    "PyRun",
    "QIF",
    "RC",
    "RandSpike",
    "Reac",
    "ReacBase",
    "SBML",
    "STDPSynHandler",
    "STDPSynapse",
    "SeqSynHandler",
    "Shell",
    "SimpleSynHandler",
    "SingleMsg",
    "SparseMsg",
    "Species",
    "SpikeGen",
    "SpikeStats",
    "Spine",
    "SpineMesh",
    "Stats",
    "SteadyState",
    "StimulusTable",
    "Stoich",
    "Streamer",
    "SymCompartment",
    "SynChan",
    "SynHandlerBase",
    "Synapse",
    "Table",
    "Table2",
    "TableBase",
    "TimeTable",
    "VClamp",
    "about",
    "Variable",
    "VectorTable",
    "ZombieBufPool",
    "ZombieCaConc",
    "ZombieCompartment",
    "ZombieEnz",
    "ZombieFunction",
    "ZombieHHChannel",
    "ZombieMMenz",
    "ZombiePool",
    "ZombieReac",
    "_moose",
    "chemMerge",
    "chemUtil",
    "connect",
    "copy",
    "delete",
    "doc",
    "element",
    "exists",
    "fixXreacs",
    "genesis",
    "getCwe",
    "getField",
    "getFieldDict",
    "getFieldNames",
    "le",
    "listmsg",
    "loadModelInternal",
    "melement",
    "mergeChemModel",
    "addChemSolver",
    "deleteChemSolver",
    "readNML2",
    "readSBML",
    "writeKkit",
    "writeNML2",
    "writeSBML",
    "moose_constants",
    "move",
    "print_utils",
    "pwe",
    "ce",
    "pydoc",
    "rand",
    "reinit",
    "seed",
    "setClock",
    "setCwe",
    "showfield",
    "showmsg",
    "start",
    "stop",
    "useClock",
    "utils",
    "vec",
    "version",
    "wildcardFind",
]


def test_attribs():
    global attribs
    for at in attribs:
        assert hasattr(moose, at), "%s not found" % at


if __name__ == "__main__":
    test_attribs()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2013, NCBS Bangalore"
__credits__          = ["NCBS Bangalore", "Bhalla Lab"]
__license__          = "GPL"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import numpy as np
import matplotlib.pyplot as plt
import moose
import moose.print_utils as pu
import re
from collections import OrderedDict

def plotAscii(yvec, xvec = None, file=None):
    """Plot two list-like object in terminal using gnuplot.
    If file is given then save data to file as well.
    """
    if xvec is None:
        plotInTerminal(yvec, list(range( len(yvec))), file=file)
    else:
        plotInTerminal(yvec, xvec, file=file)

def plotInTerminal(yvec, xvec = None, file=None):
    '''
    Plot given vectors in terminal using gnuplot.

    If file is not None then write the data to a file.
    '''
    import subprocess
    g = subprocess.Popen(["gnuplot"], stdin=subprocess.PIPE)
    g.stdin.write("set term dumb 100 25\n")
    g.stdin.write("plot '-' using 1:2 title '{}' with linespoints\n".format(file))
    if file:
        saveAsGnuplot(yvec, xvec, file=file)
    for i,j in zip(xvec, yvec):
        g.stdin.write("%f %f\n" % (i, j))
    g.stdin.write("\n")
    g.stdin.flush()

def xyToString( yvec, xvec, sepby = ' '):
    """ Given two list-like objects, returns a text string.
    """
    textLines = []
    for y, x in zip( yvec, xvec ):
        textLines.append("{}{}{}".format(y, sepby, x))
    return "\n".join(textLines)


def saveNumpyVec( yvec, xvec, file):
    """save the numpy vectors to a data-file

    """
    if file is None:
        return
    print(("[INFO] Saving plot data to file {}".format(file)))
    textLines = xyToString( yvec, xvec)
    with open(file, "w") as dataF:
        dataF.write(textLines)

def saveAsGnuplot( yvec, xvec, file):
    ''' Save the plot as stand-alone gnuplot script '''
    if file is None:
        return
    print(("[INFO] Saving plot data to a gnuplot-script: {}".format(file)))
    dataText = xyToString( yvec, xvec )
    text = []
    text.append("#!/bin/bash")
    text.append("gnuplot << EOF")
    text.append("set term post eps")
    text.append("set output \"{0}.eps\"".format(file))
    text.append("plot '-' using 0:1 title '{0}'".format(file))
    text.append(dataText)
    text.append("EOF")
    with open(file+".gnuplot","w") as gnuplotF:
        gnuplotF.write("\n".join(text))

def scaleVector(vec, scaleF):
    """ Scale a vector by a factor """
    if scaleF == 1.0 or scaleF is None:
        return vec
    else:
        return [ x*scaleF for x in vec ]

def scaleAxis(xvec, yvec, scaleX, scaleY):
    """ Multiply each elements by factor """
    xvec = scaleVector( xvec, scaleX )
    yvec = scaleVector( yvec, scaleY )
    return xvec, yvec

def reformatTable(table, kwargs):
    """ Given a table return x and y vectors with proper scaling """
    clock = moose.Clock('/clock')
    if type(table) == moose.Table:
        vecY = table.vector
        vecX = np.arange(0, clock.currentTime, len(vecY))
    elif type(table) == tuple:
        vecX, vecY = table
    return (vecX, vecY)

def plotTable(table, **kwargs):
    """Plot a given table. It plots table.vector

    This function can scale the x-axis. By default, y-axis and x-axis scaling is
    done by a factor of 1.

    Pass 'xscale' and/or 'yscale' argument to function to modify scales.

    """
    if not type(table) == moose.Table:
        msg = "Expected moose.Table, got {}".format( type(table) )
        raise TypeError(msg)

    vecX, vecY = reformatTable(table, kwargs)
    plt.plot(vecX, vecY, label = kwargs.get('label', ""))
    # This may not be available on older version of matplotlib.
    try:
        plt.legend(loc='best', framealpha=0.4)
    except:
        plt.legend(loc='best')

def plotTables(tables, outfile=None, **kwargs):
    """Plot a dict of tables.

    Each axis will be labeled with dict keys. The dict values must be
    moose.Table/moose.Table2.

    :param outfile: Default `None`. If given, plot will be saved to this filepath.
    :param grid: A tuple with (cols, rows), default is (len(tables)//2+1, 2)
    :param figsize: Size of figure (W, H) in inches. Default (10, 1.5*len(tables))
    """
    assert type(tables) == dict, "Expected a dict of moose.Table"
    plt.figure(figsize=kwargs.get('figsize', (10, 1.5*len(tables))))
    subplot = kwargs.get('subplot', True)
    gridSize = kwargs.get('grid', (len(tables)//2+1, 2))
    for i, tname in enumerate(tables):
        if subplot:
            assert gridSize[0] <= 9
            plt.subplot(100*gridSize[0]+10*gridSize[1]+(i+1))
        yvec = tables[tname].vector
        xvec = np.linspace(0, moose.Clock('/clock').currentTime, len(yvec))
        plt.plot(xvec, yvec, label=tname)

        # This may not be available on older version of matplotlib.
        try:
            plt.legend(loc='best', framealpha=0.4)
        except:
            plt.legend(loc = 'best')

    plt.tight_layout()
    if outfile:
        pu.dump("PLOT", "Saving plots to file {}".format(outfile))
        try:
            plt.savefig(outfile, transparent=True)
        except Exception as e:
            pu.dump("WARN", "Failed to save figure. Errror %s"%e)
            plt.show()
    else:
        plt.show()

def plotVector(vec, xvec = None, **options):
    """plotVector: Plot a given vector. On x-axis, plot the time.

    :param vec: Given vector.
    :param **kwargs: Optional to pass to maplotlib.
    """

    assert type(vec) == np.ndarray, "Expected type %s" % type(vec)
    legend = options.get('legend', True)

    if xvec is None:
        clock = moose.Clock('/clock')
        xx = np.linspace(0, clock.currentTime, len(vec))
    else:
        xx = xvec[:]

    assert len(xx) == len(vec), "Expecting %s got %s" % (len(vec), len(xvec))

    plt.plot(xx, vec, label=options.get('label', ''))
    if legend:
        # This may not be available on older version of matplotlib.
        try:
            plt.legend(loc='best', framealpha=0.4)
        except:
            plt.legend(loc='best')

    if xvec is None:
        plt.xlabel('Time (sec)')
    else:
        plt.xlabel(options.get('xlabel', ''))

    plt.ylabel = options.get('ylabel', '')
    plt.title(options.get('title', ''))

    if(options.get('legend', True)):
        try:
            plt.legend(loc='best', framealpha=0.4, prop={'size' : 9})
        except:
            plt.legend(loc='best', prop={'size' : 9})


def saveRecords(records, xvec = None, **kwargs):
    """saveRecords
    Given a dictionary of data with (key, numpy array) pair, it saves them to a
    file 'outfile'

    :param outfile
    :param dataDict:
    :param **kwargs:
        comment: Adds comments below the header.
    """
    if len(records) == 0:
        pu.warn("No data in dictionary to save.")
        return False

    outfile = kwargs.get('outfile', 'data.moose')
    clock = moose.Clock('/clock')
    assert clock.currentTime > 0
    yvecs = [ ]
    text = "time," + ",".join([ str(x) for x in records ])
    for k in records:
        try:
            yvec = records[k].vector
        except AttributeError as e:
            yevc = records[k]
        yvecs.append(yvec)
    xvec = np.linspace(0, clock.currentTime, len(yvecs[0]))
    yvecs = [ xvec ] + yvecs
    if kwargs.get('comment', ''):
        text += ("\n"  + kwargs['comment'] )
    np.savetxt(outfile, np.array(yvecs).T, delimiter=',' , header = text)
    pu.info("Done writing data to %s" % outfile)

def plotRecords(records, xvec = None, **kwargs):
    """Wrapper
    """
    dataDict = OrderedDict( )
    try:
        for k in sorted(records.keys(), key=str.lower):
            dataDict[k] = records[k]
    except Exception:
        dataDict = records

    outfile = kwargs.get('outfile', None)
    subplot = kwargs.get('subplot', False)
    filters = [ x.lower() for x in kwargs.get('filter', [])]

    plt.figure(figsize=(10, 1.5*len(dataDict)))
    #plt.rcParams.update( { 'font-size' : 10 } )
    for i, k in enumerate(dataDict):
        pu.info("+ Plotting for %s" % k)
        plotThis = False
        if not filters: plotThis = True
        for accept in filters:
            if accept in k.lower():
                plotThis = True
                break

        if plotThis:
            if not subplot:
                yvec = dataDict[k].vector
                plotVector(yvec, xvec, label=k, **kwargs)
            else:
                plt.subplot(len(dataDict), 1, i+1)
                yvec = dataDict[k].vector
                plotVector(yvec, xvec, label=k, **kwargs)

    # title in Image.
    if 'title' in kwargs:
        plt.title(kwargs['title'])

    if subplot:
        try:
            plt.tight_layout()
        except: pass

    if outfile:
        pu.info("Writing plot to %s" % outfile)
        plt.savefig("%s" % outfile, transparent=True)
    else:
        plt.show()
    plt.close( )


def plotTablesByRegex( regex = '.*', **kwargs ):
    """plotTables Plot all moose.Table/moose.Table2 matching given regex. By
    default plot all tables. Table names must be unique. Table name are used as
    legend.

    :param regex: Python regular expression to be matched.
    :param **kwargs:
        - subplot = True/False; if True, each Table is plotted in a subplot.
        - outfile = filepath; If given, plot will be saved to this path.
    """

    tables = moose.wildcardFind( '/##[TYPE=Table]' )
    tables += moose.wildcardFind( '/##[TYPE=Table2]' )
    toPlot = OrderedDict( )
    for t in sorted(tables, key = lambda x: x.name):
        if re.search( regex, t.name ):
            toPlot[ t.name ] = t
    return plotRecords( toPlot, None, **kwargs )



# -*- coding: utf-8 -*-
# Issue #93 on moose-core

import numpy as np
import matplotlib.pyplot as plt
import moose
import os
import sys

dt = 10e-6

d = os.path.split( os.path.abspath( __file__ ) )[0]

def loadAndRun(solver=True):
    simtime = 500e-3
    model = moose.loadModel(os.path.join(d, '../data/h10.CNG.swc'), '/cell')
    comp = moose.element('/cell/apical_e_177_0')
    soma = moose.element('/cell/soma')
    for i in range(10):
        moose.setClock(i, dt)
    if solver:
        solver = moose.HSolve('/cell/solver')
        solver.target = soma.path
        solver.dt = dt
    stim = moose.PulseGen('/cell/stim')
    stim.delay[0] = 50e-3
    stim.delay[1] = 1e9
    stim.level[0] = 1e-9
    stim.width[0] = 2e-3
    moose.connect(stim, 'output', comp, 'injectMsg')
    tab = moose.Table('/cell/Vm')
    moose.connect(tab, 'requestOut', comp, 'getVm')
    tab_soma = moose.Table('/cell/Vm_soma')
    moose.connect(tab_soma, 'requestOut', soma, 'getVm')
    moose.reinit()
    print('[INFO] Running for %s' % simtime)
    moose.start(simtime )
    vec = tab_soma.vector
    moose.delete( '/cell' )
    return vec

def main( ):
    eeVec = loadAndRun( False )
    hsolveVec = loadAndRun( True )
    clk = moose.Clock( '/clock' )
    print( '[DEBUG] Total entries %s' % len( eeVec ))
    t = np.linspace(0, clk.currentTime, len( eeVec ))
    plt.plot(t, eeVec, label = 'ee' )
    plt.plot( t, hsolveVec, label = 'hsolve' )
    outfile = '%s.png' % sys.argv[0]
    plt.legend( )
    plt.savefig( outfile )
    print( '[INFO] Wrote results to %s' % outfile )
    return 1

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2019-, Dilawar Singh"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"

import moose
import numpy as np
print( "[INFO ] Using moose %s form %s" % (moose.version(), moose.__file__) )

def create_func( funcname, expr ):
    f = moose.Function( funcname )
    f.expr = expr
    t = moose.Table( funcname + 'tab' )
    t.connect('requestOut', f, 'getValue')
    moose.setClock( f.tick, 0.1)
    moose.setClock( t.tick, 0.1)
    return f, t

def test_function():
    nsteps = 5
    simtime = nsteps
    dt = 1.0
    fn1 = moose.Function('/fn1', expr = 'B+A+y0+y1')
    inputs = np.arange(0, nsteps+1, 1.0)
    x0 = moose.StimulusTable('/x0', vector=inputs, startTime = 0.0
            , stopTime = simtime, stepPosition = 0.0)
    print('x0', x0.vector)
    inputs /= 10
    x1 = moose.StimulusTable('/x1', vector=inputs, startTime=0.0
            , stopTime=simtime, stepPosition=0.0, k = 11)
    print('x1', x1.vector)
    inputs /= 10
    y0 = moose.StimulusTable('/y0', vector=inputs, startTime=0.0
            , stopTime=simtime, stepPosition=0.0)
    print('y0', y0.vector)
    inputs /= 10
    y1 = moose.StimulusTable('/y1', vector=inputs, startTime=0.0
            , stopTime=simtime, stepPosition=0.0)
    print('y1', y1.vector)
    x0.connect('output', fn1['A'], 'input')
    x1.connect('output', fn1['B'], 'input')
    fn1.connect('requestOut', y0, 'getOutputValue')
    fn1.connect('requestOut', y1, 'getOutputValue')

    z1 = moose.Table('/z1')
    z1.connect('requestOut', fn1, 'getValue')
    for ii in range(32):
        moose.setClock(ii, dt)
    moose.reinit()
    moose.start(simtime)
    expected = [0, 1.1, 2.211, 3.322, 4.433, 5.544]
    assert np.allclose(z1.vector, expected), "Excepted %s, got %s" % (expected, z1.vector )
    print( 'Passed order vars' )


def test_creation():
    a1 = moose.Neutral('a')
    a2 = moose.Neutral('//a')
    a3 = moose.Neutral('/.//a/')
    assert a1 == a2 == a3
    assert a1.path == a2.path == a3.path == '/a[0]', a1.path
    a3 = moose.Neutral('a2')
    assert a3 != a1

def test_print():
    a = moose.Function('f', expr='sqrt(sin(x0))+cos(x1)^2+ln(20+sin(t))')
    # Get equivalent sympy expression. And do whatever you like with it.
    try:
        sympyExpr = a.sympyExpr()
        print('Sympy expression is:', sympyExpr)

        # Or pretty-print using sympy
        a.printUsingSympy()
    except ImportError:
        pass

def test_basic():
    f = moose.Function('f1')
    f.expr = 'A+B+sqrt(B)+zhadu(q)'
    print(f)

def main():
    test_function()
    test_basic()
    test_creation()
    test_print()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# test_function.py ---
# Filename: test_function.py
# Description:
# Author: subha
# Maintainer: Dilawar Singh <diawars@ncbs.res.in>
# Created: Sat Mar 28 19:34:20 2015 (-0400)
# Version:

from __future__ import print_function
import numpy as np
import moose
print( "[INFO ] Using moose %s form %s" % (moose.version(), moose.__file__) )

def create_func( funcname, expr ):
    f = moose.Function( funcname )
    f.expr = expr
    t = moose.Table( funcname + 'tab' )
    moose.connect( t, 'requestOut', f, 'getValue' )
    moose.setClock( f.tick, 0.1)
    moose.setClock( t.tick, 0.1)
    return f, t

def test_var_order():
    """The y values are one step behind the x values because of
    scheduling sequences"""
    nsteps = 5
    simtime = nsteps
    dt = 1.0
    # fn0 = moose.Function('/fn0')
    fn1 = moose.Function('/fn1')
    print(fn1.x.num)
    fn1.x.num = 2
    fn1.expr = 'y1+y0+x1+x0'
    fn1.mode = 1
    inputs = np.arange(0, nsteps+1, 1.0)
    x0 = moose.StimulusTable('/x0')
    x0.vector = inputs
    x0.startTime = 0.0
    x0.stopTime = simtime
    x0.stepPosition = 0.0
    print('x0', x0.vector)
    inputs /= 10
    x1 = moose.StimulusTable('/x1')
    x1.vector = inputs
    x1.startTime = 0.0
    x1.stopTime = simtime
    x1.stepPosition = 0.0
    print('x1', x1.vector)
    inputs /= 10
    y0 = moose.StimulusTable('/y0')
    y0.vector = inputs
    y0.startTime = 0.0
    y0.stopTime = simtime
    y0.stepPosition = 0.0
    print('y0', y0.vector)
    inputs /= 10
    y1 = moose.StimulusTable('/y1')
    y1.vector = inputs
    print('y1', y1.vector)
    y1.startTime = 0.0
    y1.startTime = 0.0
    y1.stopTime = simtime
    y1.stepPosition = 0.0
    #  print( fn1, type(fn1) )
    #  print( moose.showmsg(fn1.x) )
    moose.connect(x0, 'output', fn1.x[0], 'input')
    moose.connect(x1, 'output', fn1.x[1], 'input')
    moose.connect(fn1, 'requestOut', y0, 'getOutputValue')
    moose.connect(fn1, 'requestOut', y1, 'getOutputValue')
    z1 = moose.Table('/z1')
    moose.connect(z1, 'requestOut', fn1, 'getValue')
    for ii in range(32):
        moose.setClock(ii, dt)
    moose.reinit()
    moose.start(simtime)
    expected = [0, 1.1, 2.211, 3.322, 4.433, 5.544]
    print('sum: ', x0.vector + x1.vector + y0.vector  + y1.vector)
    print('got', z1.vector)
    assert np.allclose(z1.vector, expected), "Excepted %s, got %s" % (expected, z1.vector )
    print( 'Passed order vars' )

def test_t( ):
    f, t = create_func( 'funct', 't/2.0')
    moose.reinit()
    moose.start(1)
    y = t.vector
    d = np.diff( y[1:] ) 
    assert np.mean(d) == d[0]
    print( 'Passed t/2' )

def test_trig( ):
    f, t = create_func('func2', '(sin(t)^2+cos(t)^2)-1')
    moose.reinit()
    moose.start( 1 )
    y = t.vector
    print(y)
    assert np.isclose(np.mean(y),  0.0), np.mean(y)
    assert np.isclose(np.std(y), 0.0), np.std(y)
    print( 'Passed sin^2 x + cos^x=1' )

def test_rand( ):
    moose.seed( 10 )
    f, t = create_func( 'random', 'rnd()')
    moose.reinit()
    moose.start(1000)
    print(np.mean(t.vector), np.std(t.vector), 'xxx')
    assert abs(np.mean(t.vector) - 0.5) < 0.01
    assert np.std(t.vector) < 0.3
##    expected = [0.29876116, 0.49458993, 0.83191136, 0.02517173, 0.26556613,
##            0.15037787, 0.81660184, 0.89081653, 0.03061665, 0.72743551, 0.13145815]
##    assert np.isclose(t.vector, expected ).all(), "Expected %s, got %s" % (
##            expected, t.vector)
##    print( 'Passed test random' )

def test_fmod( ):
    f, t = create_func( 'fmod', 'fmod(t, 2)' )
    moose.reinit()
    moose.start(20)
    y = t.vector
    print(y)
    assert (np.fmod(y, 2) == y).all()
    assert(np.isclose(np.max(y), 1.9)), "Expected 1.9 got %s" % np.max(y)
    assert(np.isclose(np.min(y), 0.0)), "Expected 0.0 got %s" % np.min(y)
    print('Passed fmod(t,2)')


if __name__ == '__main__':
    test_var_order()
    test_t()
    test_trig()
    test_rand()
    test_fmod()



import matplotlib as mpl
mpl.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import moose
print("[INFO ] MOOSE version=%s, loc=%s" % (moose.version(), moose.__file__))
import rdesigneur as rd

expected = (-0.051693861353435865, 0.004062563722923687 
        , np.array([7, 8, 11, 17, 34, 1852, 49, 31, 34, 958])
        , np.array([-0.065, -0.06309117, -0.06118235, -0.05927352, -0.05736469,
                -0.05545587, -0.05354704, -0.05163822, -0.04972939, -0.04782056,
                -0.04591174]))

def test_msgs():
    if moose.exists('/model'):
        moose.delete('/model')
    rdes = rd.rdesigneur(
        stimList = [['soma', '1', '.', 'inject', '(t>0.1 && t<0.2) * 2e-8' ]],
        plotList = [['soma', '1', '.', 'Vm', 'Soma membrane potential']]
    )
    rdes.buildModel()
    msgs = moose.listmsg(rdes.soma)
    assert [msg.isA['Msg'] for msg in msgs] == [True, True, True]
    assert len(msgs) == 3, msgs

def test_current_pulse():
    """Test current pulse.
    """
    rdes = rd.rdesigneur(
        stimList = [['soma', '1', '.', 'inject', '(t>0.1 && t<0.2) * 2e-8' ]],
        plotList = [['soma', '1', '.', 'Vm', 'Soma membrane potential']]
    )
    rdes.buildModel()
    moose.reinit()
    moose.start(0.3)
    rdes.display(block=False)
    y = moose.wildcardFind('/##[TYPE=Table]')[0].vector
    ym, ys = y.mean(), y.std()
    X = np.histogram(y)
    assert np.allclose(ym, expected[0])
    assert np.allclose(ys, expected[1])
    assert np.allclose(X[0], expected[2])
    assert np.allclose(X[1], expected[3])


if __name__ == '__main__':
    test_current_pulse()
    test_msgs()



# -*- coding: utf-8 -*-
import moose
import re

def fixPath(path):
    path = re.sub(r'/+', '/', path)
    return path

def test_path():
    paths = [ '/a'
            , '//a'
            , '/a/b'
            , '/a/b/'
            , '//a//b/////'
            , '/a/./b'
            , '///a/././b'
            ]

    expectedPath = set([fixPath(p) for p in paths])
    print(expectedPath)

    for p in paths:
        print("-- Creating %s" % p)
        if not moose.exists(p):
            print('exists')
            p = moose.Neutral(p)
        else:
            p = moose.element(p)
        print(p)

    foundPath = []
    for p in moose.wildcardFind('/##'):
        if "/a" in p.path:
           foundPath.append(p.path)

    testFailed = False
    for f in foundPath:
        f = re.sub(r'\[\d+\]', '', f)
        if f not in expectedPath:
            testFailed = True

    assert not testFailed, "Test failed"


def main():
    test_path()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# compare_moose_nrn.py ---
#
# Filename: compare_moose_nrn.py
# Description:
# Author: subha
# Maintainer:
# Created: Sat Apr 30 02:43:29 2016 (-0400)
# Version:
# Last-Updated: Sat Apr 30 03:01:04 2016 (-0400)
#           By: subha
#     Update #: 19
# URL:
# Keywords:
# Compatibility:
#
#

# Commentary:
#
#
#
#

# Change log:
#
#
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA.
#
#

# Code:

import pylab as pl

nrn = pl.loadtxt('h10.CNG.nrn.txt')
print(nrn.shape)
ee = pl.loadtxt('h10.CNG.moose.ee.txt')
hs = pl.loadtxt('h10.CNG.moose.hsolve.txt')
pl.plot(nrn[:, 0], nrn[:, 1], label='NEURON soma')
pl.plot(nrn[:, 0], nrn[:, 2], label='NEURON apical[90]')
pl.plot(ee[:, 0]*1e3, ee[:, 1]*1e3, label='MOOSE EE soma')
pl.plot(ee[:, 0]*1e3, ee[:, 2]*1e3, label='MOOSE EE apical_e_177_0')
pl.plot(hs[:, 0]*1e3, hs[:, 1]*1e3, label='MOOSE HSolve soma')
pl.plot(hs[:, 0]*1e3, hs[:, 2]*1e3, label='MOOSE HSolve apical_e_177_0')
pl.legend()
pl.show()


#
# compare_moose_nrn.py ends here



__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2019-, Dilawar Singh"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"

import doctest
import moose

print("[INFO ] Using moose from", moose.about())

def test_showfield_func():
    """Show field of a function.
    >>> test_showfield_func()   #doctest: +NORMALIZE_WHITESPACE
    [/dadada[0]]
    allowUnknownVariable    =True
    className               =Function
    derivative              =nan
    doEvalAtReinit          =False
    dt                      =0.1
    expr                    =0
    fieldIndex              =0
    idValue                 =448
    independent             =t
    index                   =0
    mode                    =1
    name                    =dadada
    numData                 =1
    numField                =1
    numVars                 =0
    path                    =/dadada[0]
    rate                    =0.0
    tick                    =12
    useTrigger              =False
    value                   =0.0
    """
    f = moose.Function('/dadada')
    moose.showfield(f)

if __name__ == '__main__':
    import doctest
    doctest.testmod()



# savemodel.py ---

# Filename: savemodel.py
# Description:
# Author: Subhasis Ray
# Maintainer:
# Created: Wed Oct 29 00:08:50 2014 (+0530)
# Version:
# Last-Updated:Thr Dec 23 16:31:00 2015 (+0530)
#           By:
#     Update #: 0
# URL:
# Keywords:
# Compatibility:

import os
import traceback
import moose

cwd = os.path.dirname( os.path.realpath( __file__ ) )

def test_kkit():
    """ 
    The script demonstates to convert Chemical (Genesis) file back to Genesis 
    file using moose 
    """
    moose.loadModel(os.path.join( cwd, '../data/reaction.g'), '/model')
    written = moose.writeKkit('/model', 'testsave.g')
    print( written )

if __name__ == '__main__':
    test_kkit()



# -*- coding: utf-8 -*-
"""count.py:

Counting related function goes here.

"""

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2015, Dilawar Singh and NCBS Bangalore"
__credits__          = ["NCBS Bangalore"]
__license__          = "GNU GPL"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"


import numpy as np

def locate_spike_in_spike_train( vec, threshold ):
    """Return a list of instance where a spike has occured in vec """

    vec = np.array( vec )
    nSpikes = 0
    spikeBegin, spikeEnds = False, False
    spikeList = []
    for i, x in enumerate(vec):
        if x > threshold:
            if not spikeBegin:
                spikeBegin, spikeEnds = True, False
            else: pass
        else:
            if spikeBegin:
                spikeEnds, spikeBegin = True, False
                spikeList.append(i)
                nSpikes += 1
    return nSpikes, spikeList

def spike_train_simple_stat( vec, threshold = 0.0, **kwargs ):
    """Compute number of spikes, mean and variance of spike intervals in a given
    vec.

    """
    vec = np.array(vec)
    result = {}
    nSpikes, spikeList = locate_spike_in_spike_train(vec, threshold)
    if len(spikeList) > 1:
        diffIndex = np.diff(spikeList)
        meanSteps = np.mean(diffIndex)
        varSteps = np.var(diffIndex)
    else:
        meanSteps = -1
        varSteps = -1
    result['number of spikes'] = nSpikes
    result['mean spike-interval'] = meanSteps
    result['variance spike-interval'] = varSteps
    result['min'] = vec.min()
    result['max'] = vec.max()
    result['variance'] = np.var(vec)
    return result



# -*- coding: utf-8 -*-
from __future__ import print_function, division

################################################################
# This program is part of 'MOOSE', the
# Messaging Object Oriented Simulation Environment.
#           Copyright (C) 2015 Upinder S. Bhalla. and NCBS
# It is made available under the terms of the
# GNU Lesser General Public License version 2.1
# See the file COPYING.LIB for the full notice.
#
# rxdSpineSize.py: Builds a cell with spines and a propagating reaction
# wave. Products diffuse into the spine and cause it to get bigger.
#################################################################

# Cdc42 activation profile (spatial and time) has to be accounted,
# Tension modification
# Modification logs:
#     Model is developed by Vinu Verghese.
#     Modified by Dilawar Singh <dilawar.s.rajput@gmail.com> for testing.

import os
import numpy as np
import moose
print("[INFO] MOOSE: %s, version:%s" % (moose.__file__, moose.__version__))
import sys
import rdesigneur as rd
import time
import xml.etree.ElementTree as ET


if sys.version_info.major == 2:
    print("[INFO ] this test works with python3 only.")
    quit(0)

try:
    import moose.SBML
except Exception as e:
    print(e)
    print("libsbml-python is not found. Disabling tests.")
    quit(0)

sdir_ = os.path.dirname(os.path.realpath(__file__))
args_ = [
    '', 1e-6, 0.018, 1, 20e-12, 1e-12, 100, 102, 105, 4, 20, 10, 1.0, 0.5,
    0.05, 0.0, 0.0055, 0.1e-14
]

PI = 3.141592653
scaling_ = 10.0
RM = 1.0 / scaling_
RA = 1.0 * scaling_
CM = 0.01 * scaling_
loc1 = str(args_[6])
loc2 = str(args_[7])
loc3 = str(args_[8])
frameruntime = 1.0
diffConst = 5e-10
dendLen = 0.25e-6
comptLen = 1e-6
diffLen = 2.0e-6
dendDia = 1e-6
somaDia = 1e-6
concInit = 0.1  # 10 micromolar
spineSpacing = float(args_[1])
spineSpacingDistrib = 1e-6
spineSize = 1.0
spineSizeDistrib = 0.5
spineAngle = np.pi / 2.0
spineAngleDistrib = 0.0
numDendSegments = 100


def computeTP(t, distS, SourceC):
    print("[INFO ] Calling computeTP", end=' ....')
    sys.stdout.flush()
    conv = moose.element('/model/chem/dend/conv')
    w = len(conv.vec.n)
    h = w
    v2d = [[0.0 for x in range(w)] for y in range(h)]
    mm = moose.vec('/model/chem/dend/mesh')
    diffL = moose.element('/model/chem/dend').diffLength
    print(conv.name, conv.vec[0].nInit, 'diff lenght %s' % diffL)
    listL = [[distS, t]]
    SourceC = SourceC
    extraDiff = 0.05e-12
    for j in listL:
        for i in range(len(conv.vec.n)):
            for k in range(len(conv.vec.conc)):
                v2d[i][k] = diffL**2.0 * (
                    SourceC /
                    (4.0 * np.pi * j[1] * np.sqrt(extraDiff * extraDiff))
                ) * np.exp(-(mm[i].Coordinates[0]**2.0) /
                           (4 * extraDiff * j[1]) - (mm[k].Coordinates[0]**2) /
                           (4 * extraDiff * j[1]))
    tempRec = []
    if True:
        for i in range(len(conv.vec.n)):
            tempRec.append(v2d[i][distS])
    print("  .... done")
    sys.stdout.flush()
    return tempRec


def pert_int(posPer, SourceC):
    print('In pert_int')
    timePer = int(args_[9])
    distS = int(args_[10])
    Ca = moose.element('/model/chem/dend/Ca')
    conv = moose.element('/model/chem/dend/conv')
    rec = moose.element('/model/chem/dend/IRSpGr/rec')
    Ca_vec = moose.vec('/model/chem/dend/Ca').conc
    tempRec = computeTP(timePer, distS, SourceC)
    print(len(tempRec))
    sys.stdout.flush()
    moose.element('/model/chem/dend/IRSpGr/rec').vec[posPer].conc += tempRec[0]
    for j in range(posPer + 1, len(Ca_vec)):
        print(j, end=' ')
        sys.stdout.flush()
        #rec.vec[j].conc=tempRec[j-posPer]
        moose.element('/model/chem/dend/IRSpGr/rec').vec[j].conc += tempRec[
            j - posPer]
        if int(posPer * 2 - j) >= 0:
            #rec.vec[posPer*2-j].conc=tempRec[j-posPer]
            moose.element('/model/chem/dend/IRSpGr/rec').vec[
                posPer * 2 - j].conc += tempRec[j - posPer]
    moose.element('/model/chem/dend/IRSpGr/rec').conc = moose.element(
        '/model/chem/dend/IRSpGr/rec').conc / 2

    print('PERTURBED')

    return distS, timePer


def makeCellProto(name):
    elec = moose.Neuron('/library/' + name)
    ecompt = []
    #soma = rd.buildCompt( elec, 'soma', dx=somaDia, dia=somaDia, x=-somaDia, RM=RM, RA=RA, CM=CM )
    dend = rd.buildCompt(elec,
                         'dend',
                         dx=dendLen,
                         dia=dendDia,
                         x=0,
                         RM=RM,
                         RA=RA,
                         CM=CM)
    #moose.connect( soma, 'axial', dend, 'raxial' )
    prev = dend
    x = dendLen
    for i in range(numDendSegments):
        compt = rd.buildCompt(elec,
                              'dend' + str(i),
                              dx=dendLen,
                              dia=dendDia,
                              x=x,
                              RM=RM,
                              RA=RA,
                              CM=CM)
        moose.connect(prev, 'axial', compt, 'raxial')
        prev = compt
        x = x + dendLen
    elec.buildSegmentTree()


def makeDendProto(name):
    dend = moose.Neuron('/library/dend')
    prev = rd.buildCompt(dend,
                         'soma',
                         RM=RM,
                         RA=RA,
                         CM=CM,
                         dia=2.0e-06,
                         x=0,
                         dx=comptLen)
    x = comptLen
    y = 0.0
    comptDia = 1e-06

    for i in range(numDendSegments):
        dx = comptLen
        dy = 0
        compt = rd.buildCompt(dend,
                              'dend' + str(i),
                              RM=RM,
                              RA=RA,
                              CM=CM,
                              x=x,
                              y=y,
                              dx=dx,
                              dy=dy,
                              dia=comptDia)
        moose.connect(prev, 'axial', compt, 'raxial')
        prev = compt
        x += dx
        y += dy


def makeChemProto(name='hydra'):
    moose.Neutral('/library/')
    meshName = '/library/' + name
    moose.SBML.mooseReadSBML(os.path.join(sdir_, '..', 'data', 'c_m.xml'),
                             meshName)
    Rad = moose.element('/library/hydra/dend/IRSpGr/Rad')

    I_C = moose.element(meshName + '/dend/IRSpGr/IRSp53')
    Ca = moose.element(meshName + '/dend/Ca')
    rec = moose.Pool(meshName + '/dend/IRSpGr/rec')
    conv = moose.Pool(meshName + '/dend/conv')
    Cdc42 = moose.element(meshName + '/dend/IRSpGr/Cdc42')
    Cdc42_m = moose.element(meshName + '/dend/IRSpGr/Cdc42_m')
    Cdc42_GDP = moose.element(meshName + '/dend/Cdc42_GDP')
    Rac_GTP = moose.element(meshName + '/dend/Rac_GTP')
    Rac_m = moose.element(meshName + '/dend/IRSpGr/Rac_m')
    Rac_GDP = moose.element(meshName + '/dend/Rac_GDP')
    Eps8 = moose.element(meshName + '/dend/IRSpGr/Eps8')
    I_Cact = moose.element(meshName + '/dend/IRSpGr/IRSp53_a')
    I_M = moose.element(meshName + '/dend/IRSpGr/IRSp53_m')
    Idimer = moose.element(meshName + '/dend/IRSpGr/IRSp53_dimer')
    recBAR = moose.Function(meshName + '/dend/IRSpGr/recBAR')
    gradBAR = moose.Function(meshName + '/dend/IRSpGr/gradBAR')
    Radfun = moose.Function(meshName + '/dend/IRSpGr/Radfun')
    Ipool = moose.element(meshName + '/dend/IRSpGr/Ipool')
    sort = moose.element(meshName + '/dend/IRSpGr/sort')
    detach = moose.element(meshName + '/dend/IRSpGr/detach')
    curv_IRSp53 = moose.element(meshName + '/dend/IRSpGr/curv_IRSp53')
    tube_IRSp53 = moose.element(meshName + '/dend/IRSpGr/tube_IRSp53')
    to_tube_Reac = moose.element(meshName + '/dend/IRSpGr/to_tube_Reac')
    Cdc42_breac = moose.element(meshName + '/dend/Cdc42_GTP_GDP_Reac')
    Tiam_breac = moose.element(meshName + '/dend/Tiam_Rev_Reac')
    store_tube = moose.Pool(meshName + '/dend/IRSpGr/store_tube')
    IRSp53_temp = moose.element(meshName + '/dend/IRSpGr/IRSp53_temp')
    sigma = moose.Pool(meshName + '/dend/IRSpGr/sigma')
    Kv = moose.Pool(meshName + '/dend/IRSpGr/Kv')

    dend_vasp = moose.element(meshName + '/dend/IRSpGr/VASP')

    dend_gactin = moose.element(meshName + '/dend/IRSpGr/Gactin')
    dend_factin = moose.element(meshName + '/dend/IRSpGr/Factin')
    GF_Reac = moose.element(meshName + '/dend/IRSpGr/GF_Reac')
    sort_func = moose.element(meshName + '/dend/IRSpGr/sort_func')
    breacT_fun = moose.Function(meshName + '/dend/breacT_fun')
    breacC_fun = moose.Function(meshName + '/dend/breacC_fun')
    simpleFun = moose.Function(meshName + '/dend/IRSpGr/simpleFun')
    simpleFun1 = moose.Function(meshName + '/dend/IRSpGr/simpleFun1')
    simpleFun2 = moose.Function(meshName + '/dend/IRSpGr/simpleFun2')
    Kvfun = moose.Function(meshName + '/dend/IRSpGr/Kvfun')
    recFun = moose.Function(meshName + '/dend/IRSpGr/recFun')
    simpleFun2 = moose.Function(meshName + '/dend/IRSpGr/simpleFun2')
    simpleFun3 = moose.Function(meshName + '/dend/IRSpGr/simpleFun3')
    mod_sort = moose.element(meshName + '/dend/IRSpGr/mod_sort')
    mod_detach = moose.element(meshName + '/dend/IRSpGr/mod_detach')
    to_tube_Reac = moose.element(meshName + '/dend/IRSpGr/to_tube_Reac')

    conv_fun = moose.Function(meshName + '/dend/IRSpGr/conv_fun')
    conv_fun.expr = "1*x0*(x1-1.89191155)+1.89191155"
    Kvfun.expr = "1.0"
    moose.connect(Kvfun, 'valueOut', Kv, 'setN')

    simpleFun.expr = "1.0*x0"
    simpleFun1.expr = "1.0*x0"
    simpleFun2.expr = "0.1*x0"
    simpleFun3.expr = "1.0*x0"
    breacT_fun.expr = "5*x0"
    breacC_fun.expr = "5*x0"

    moose.connect(curv_IRSp53, 'nOut', simpleFun2.x[0], 'input')
    moose.connect(simpleFun2, 'valueOut', GF_Reac, 'setNumKf')

    moose.connect(sort_func, 'nOut', simpleFun.x[0], 'input')
    moose.connect(detach, 'nOut', simpleFun1.x[0], 'input')
    moose.connect(store_tube, 'nOut', simpleFun3.x[0], 'input')

    recFun.expr = "0.0*x0+(exp(45*54*((" + str(
        args_[14]) + "/(x0*1e9))-(1/(2.0*(x0*1e9)*(x0*1e9))))))"

    Radfun.expr = "sqrt(((45+12)*4*1e-18)/(2*(x2-0.08*(ln((((1.0*x0+1.0*x1)/(2*3.14*(0.45*0.05)))*1e-6*54))))))*((3*" + str(
        float(args_[2])) + "*x3*(x0+1.0*x1)^2)/(1.+3*" + str(float(
            args_[2])) + "*x3*(x0+1.0*x1)^2))"

    mod_sort.Kf = 0.
    mod_sort.Kb = 0.
    mod_detach.Kf = 0.
    mod_detach.Kb = 0.

    moose.connect(curv_IRSp53, 'nOut', Radfun.x[0], 'input')
    moose.connect(I_M, 'nOut', Radfun.x[1], 'input')
    #moose.connect(tube_IRSp53,'nOut',Radfun.x[2],'input')
    moose.connect(sigma, 'nOut', Radfun.x[2], 'input')
    moose.connect(Kv, 'nOut', Radfun.x[3], 'input')
    moose.connect(Radfun, 'valueOut', Rad, 'setN')

    moose.connect(rec, 'nOut', conv_fun.x[0], 'input')
    moose.connect(conv, 'nOut', conv_fun.x[1], 'input')
    moose.connect(conv_fun, 'valueOut', Ca, 'setN')

    moose.connect(simpleFun, 'valueOut', mod_sort, 'setNumKf')
    moose.connect(simpleFun1, 'valueOut', mod_detach, 'setNumKf')

    moose.connect(Rad, 'nOut', recFun.x[0], 'input')
    moose.connect(recFun, 'valueOut', Ipool, 'setN')

    ### Ipool is further copied to sort to participate in the reaction tube_reac or enz (model version 2.5.3)

    I_C.concInit = 0.0012
    file_molWt = os.path.join(sdir_, '..', 'data', 'prot_wt.xml')
    den = 6.0 * 3.14 * np.cbrt(3.0 / (4.0 * 3.14 * 0.73 * 6.02 * 1e23))
    num = 1e3 * 1e6 * 1.38 * 1e-23
    co = num / den

    # Diffconsts are set here
    if int(args_[3]) == 1:
        for prot in moose.wildcardFind(meshName + '/dend/#[ISA=Pool]'):
            mol_name = ET.parse(file_molWt).find(str(prot.name))
            print('Name is %s' % mol_name, end='. ')
            molWt = mol_name.text.split()
            if float(molWt[1]) <= 0.0:
                continue
            moose.element(prot).diffConst = co * (
                300 / (200.0 * np.cbrt(float(molWt[1]) * 1e3))) * 1e-4
            print('-- D', prot.name, moose.element(prot).diffConst)

        for prot in moose.wildcardFind(meshName +
                                       '/dend/CaMKII_gr/#[ISA=Pool]'):
            mol_name = ET.parse(file_molWt).find(str(prot.name))
            print('Name is %s' % mol_name, end='. ')
            molWt = mol_name.text.split()
            if float(molWt[1]) <= 0.0:
                continue
            moose.element(prot).diffConst = co * (
                300 / (200.0 * np.cbrt(float(molWt[1]) * 1e3))) * 1e-4
            print('-- D', prot.name, moose.element(prot).diffConst)
        for prot in moose.wildcardFind(meshName + '/dend/Ras_gr/#[ISA=Pool]'):
            mol_name = ET.parse(file_molWt).find(str(prot.name))
            print('Name is %s' % mol_name, end='. ')
            molWt = mol_name.text.split()
            if float(molWt[1]) <= 0.0:
                continue
            moose.element(prot).diffConst = co * (
                300 / (200.0 * np.cbrt(float(molWt[1]) * 1e3))) * 1e-4
            print('-- D', prot.name, moose.element(prot).diffConst)

        Ca.diffConst = float(args_[4])
        print('-- D', Ca.name, Ca.diffConst)
        print('\t==================== diffConst updated')

    print('DIFFUSION FOR IRSp53 GROUP')
    I_C.diffConst = co * (300 / (8.0 * np.cbrt(120.0 * 1e3))) * 1e-4
    print(I_C.diffConst, end=' + ')
    I_Cact.diffConst = co * (300 / (8.0 * np.cbrt(208.0 * 1e3))) * 1e-4
    print(I_Cact.diffConst)
    Eps8.diffConst = co * (300 / (200.0 * np.cbrt(92.0 * 1e3))) * 1e-4
    print(Eps8.diffConst, end=' + ')
    Cdc42.diffConst = co * (300 / (8.0 * np.cbrt(22.0 * 1e3))) * 1e-4
    Cdc42_m.diffConst = co * (300 / (200.0 * np.cbrt(22.0 * 1e3))) * 1e-4
    Cdc42_GDP.diffConst = co * (300 / (8.0 * np.cbrt(22.0 * 1e3))) * 1e-4
    print(Cdc42.diffConst, end=' + ')
    Rac_GTP.diffConst = co * (300 / (8.0 * np.cbrt(22.0 * 1e3))) * 1e-4
    Rac_m.diffConst = co * (300 / (200.0 * np.cbrt(22.0 * 1e3))) * 1e-4
    Rac_GDP.diffConst = co * (300 / (8.0 * np.cbrt(22.0 * 1e3))) * 1e-4

    I_M.diffConst = 1.0 * co * (300 / (200.0 * np.cbrt(392.0 * 1e3))) * 1e-4
    print(I_M.diffConst, end=' + ')
    Idimer.diffConst = co * (300 / (8.0 * np.cbrt(120.0 * 1e3))) * 1e-4
    print(Idimer.diffConst)
    curv_IRSp53.diffConst = float(args_[17])

    conv.diffConst = 0.0
    rec.diffConst = 0.0

    dend_gactin.diffConst = 1e-13
    dend_vasp.diffConst = 1e-13

    return meshName


def makeModel():
    moose.Neutral('/library')
    makeCellProto('cellProto')
    makeChemProto()
    rdes = rd.rdesigneur(
        useGssa=False,
        turnOffElec=False,
        chemDt=0.0025,
        diffDt=0.0025,
        chemPlotDt=0.1,
        combineSegments=False,
        stealCellFromLibrary=True,
        diffusionLength=50e-9,
        cellProto=[['cellProto', 'elec']],
        chanProto=[
            ['make_NMDA()', 'NMDA'],
            ['make_Ca_conc()', 'Ca_conc'],
        ],
        chemProto=[['hydra', 'chem']],
        chemDistrib=[["chem", "#", "install", "1"]],
        chanDistrib=[
            ['NMDA', 'dend#', 'Gbar', '1'],
            ['Ca_conc', 'dend#', 'tau', '0.033'],
        ],
        stimList=[
            [
                'dend#', '1.', 'NMDA', 'periodicsyn',
                str(args_[13]) + '*(t>0 && t<60)+' + str(args_[13]) + '*(t>' +
                str(args_[15]) + ' && t<(' + str(args_[15]) + '+60))'
            ],
        ],
        adaptorList=[['Ca_conc', 'Ca', 'dend/conv', 'conc', 0.00008, 20]],
        benchmark=True)
    moose.seed(100)
    print("[INFO ] rdes is building model ...")
    rdes.buildModel('/model')
    print("[INFO ] Model is built...")
    moose.element('/model/chem/dend/IRSpGr/Rad').nInit = 30e-9


def makePlot(name, srcVec, field):
    tab = moose.Table2('/graphs/' + name + 'Tab', len(srcVec)).vec
    for i in zip(srcVec, tab):
        moose.connect(i[1], 'requestOut', i[0], field)
    return tab


def main():
    """
    This illustrates the use of rdesigneur to build a simple dendrite with
    spines, and then to resize them using spine fields. These are the
    fields that would be changed dynamically in a simulation with reactions
    that affect spine geometry.
    In this simulation there is a propagating reaction wave using a
    highly abstracted equation, whose product diffuses into the spines and
    makes them bigger.
    """
    t0 = time.time()
    makeModel()
    print("[INFO ] Took %f sec." % (time.time() - t0))
    sys.stdout.flush()
    for i in range(11, 18):
        moose.setClock(i, 0.0025)
    moose.setClock(18, 0.0025)
    moose.element('/model/elec')
    moose.Neutral('/graphs')
    Cdcinit = moose.vec('/model/chem/dend/IRSpGr/Cdc42')
    Tiaminit = moose.vec('/model/chem/dend/Tiam1')
    Cdcinit.concInit = 0.0
    Tiaminit.concInit = 0.0

    makePlot('dend_curv_IRSp53',
             moose.vec('/model/chem/dend/IRSpGr/curv_IRSp53'), 'getN')
    makePlot('sorting', moose.vec('/model/chem/dend/IRSpGr/sort'), 'getN')
    makePlot('Ca', moose.vec('/model/chem/dend/Ca'), 'getN')
    makePlot('CaM', moose.vec('/model/chem/dend/CaM'), 'getConc')
    makePlot('CaM_GEF', moose.vec('/model/chem/dend/Ras_gr/CaM_GEF'),
             'getConc')
    makePlot('Ca4CaM', moose.vec('/model/chem/dend/CaM_Ca4'), 'getConc')
    makePlot('Cdc42', moose.vec('/model/chem/dend/IRSpGr/Cdc42'), 'getN')
    makePlot('RacGTP', moose.vec('/model/chem/dend/Rac_GTP'), 'getN')
    makePlot('Tiam', moose.vec('/model/chem/dend/Tiam1'), 'getN')
    mypyrun = moose.PyRun('/model/mypyrun')
    mypyrun.initString = """count=0"""
    mypyrun.runString = """count=count+1
radv=moose.vec('/model/chem/dend/IRSpGr/Rad').n
radm=radv[10]
output=radm
IRvec=moose.vec( '/model/chem/dend/IRSpGr/IRSp53' ).n
curv_IRSp53=moose.element( '/model/chem/dend/IRSpGr/curv_IRSp53')
Cdcvec=moose.vec( '/model/chem/dend/IRSpGr/Cdc42' ).n
if count==1:
    Cdcinit=moose.vec( '/model/chem/dend/IRSpGr/Cdc42' ).nInit
    pos=np.unravel_index(np.argmax(Cdcinit), Cdcinit.shape)
    pos=pos[0]
curv_IRSp53vec=moose.vec('/model/chem/dend/IRSpGr/curv_IRSp53').n
sigmaVec=moose.vec('/model/chem/dend/IRSpGr/sigma').n
IRSp53_mvec=moose.vec('/model/chem/dend/IRSpGr/IRSp53_m').n
Ipoolvec=moose.vec('/model/chem/dend/IRSpGr/Ipool').n
Kv=moose.vec('/model/chem/dend/IRSpGr/Kv').n
sort_func=moose.vec('/model/chem/dend/IRSpGr/sort').n
for gri in range(0,len(Cdcvec)):
   sort_func[gri]=1.0*((3*float(args_[2])*Kv[gri]*(1.0*curv_IRSp53vec[gri]+1.0*IRSp53_mvec[gri])**2)/(1.0+3.0*float(args_[2])*Kv[gri]*(1.0*curv_IRSp53vec[gri]+1.0*IRSp53_mvec[gri])**2))*sort_func[gri]
if count==1:
   maxCIR=[]
   maxIR=[]
   max_curv=[]
for gri in range(0,len(Cdcvec)):
   sigmaVec[gri]=(0.3-0.25*np.exp(-0.015*count*0.0025))*((float(args_[2])*3*curv_IRSp53vec[gri]**2)/(1+float(args_[2])*3*curv_IRSp53vec[gri]**2))+0.0055
   s_m=1-np.exp(-sigmaVec[gri]/0.3)
   sort_func[gri]=sort_func[gri]-1.6*s_m*sort_func[gri]
det_vec=moose.vec('/model/chem/dend/IRSpGr/detach').n
for gri in range(0,len(Cdcvec)):
   s_m=1-np.exp(-sigmaVec[gri]/0.3)
   det_vec[gri]=det_vec[gri]+1.6*s_m*det_vec[gri]
moose.vec('/model/chem/dend/IRSpGr/detach').n=det_vec   
temp_sort=moose.vec('/model/chem/dend/IRSpGr/sort_func').n
pref_curv=0.055
radmax=max(radv)
if count%100==0:
  print("Sum of IRSp53s")
  print(sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_dimer').n)+sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_a').n)+sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_m').n)+sum(moose.vec('/model/chem/dend/IRSpGr/curv_IRSp53').n),sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_m').n),sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_a').n),sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_dimer').n),sum(moose.vec('/model/chem/dend/IRSpGr/curv_IRSp53').n))
  sys.stdout.flush()
"""
    outputTab = moose.Table('/model/output')
    moose.connect(mypyrun, 'output', outputTab, 'input')
    moose.setClock(30, 0.0025)
    moose.setClock(10, 0.0025)
    moose.setClock(15, 0.0025)
    moose.setClock(16, 0.0025)
    mypyrun.mode = 1

    k = moose.wildcardFind('/##[TYPE=Ksolve]')[0]
    k.numThreads = 4
    print('   .... done making plots.')
    print("[INFO ] Doing moose.reinit ...")
    sys.stdout.flush()
    moose.reinit()
    print("     ... DONE REINIT")
    sys.stdout.flush()
    print('\n TEST PROPAGATING WAVES \n')
    recavec = []
    loc1_vox = (int(loc1) * dendLen) / 50e-9
    loc2_vox = (int(loc2) * dendLen) / 50e-9
    loc3_vox = (int(loc3) * dendLen) / 50e-9

    pert_int(int(loc1_vox), int(args_[11]))

    temp = moose.vec('/model/chem/dend/IRSpGr/rec').n
    recavec.append(moose.vec('/model/chem/dend/IRSpGr/rec').n)
    gap = int(loc2_vox) - int(loc1_vox)
    gap_p = int(loc3_vox) - int(loc1_vox)
    recavec.append([
        0.0 + float(args_[12]) * temp[i - gap_p]
        for i in range(0, len(moose.vec('/model/chem/dend/IRSpGr/rec').n))
    ])
    moose.vec('/model/chem/dend/IRSpGr/rec').n = [
        temp[i - gap]
        for i in range(0, len(moose.vec('/model/chem/dend/IRSpGr/rec').n))
    ]
    recavec.append(moose.vec('/model/chem/dend/IRSpGr/rec').n)
    moose.vec('/model/chem/dend/IRSpGr/rec').n = [
        temp[i - gap]
        for i in range(0, len(moose.vec('/model/chem/dend/IRSpGr/rec').n))
    ]
    moose.vec('/model/chem/dend/IRSpGr/rec'
              ).n = moose.vec('/model/chem/dend/IRSpGr/rec').n + temp
    recavec.append(moose.vec('/model/chem/dend/IRSpGr/rec').n)
    if float(args_[15]) == 0.0:
        moose.vec('/model/chem/dend/IRSpGr/rec').n = recavec[3]
    else:
        moose.vec('/model/chem/dend/IRSpGr/rec').n = recavec[0]

    allres = []
    t0 = time.time()
    for i in range(3):
        print("[INFO ] Simulation step %d/3" % (1 + i))
        moose.start(0.5)
        res = sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_dimer').n) + sum(
            moose.vec('/model/chem/dend/IRSpGr/IRSp53_a').n
        ) + sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_m').n) + sum(
            moose.vec('/model/chem/dend/IRSpGr/curv_IRSp53').n), sum(
                moose.vec('/model/chem/dend/IRSpGr/IRSp53_m').n
            ), sum(moose.vec('/model/chem/dend/IRSpGr/IRSp53_a').n), sum(
                moose.vec('/model/chem/dend/IRSpGr/IRSp53_dimer').n), sum(
                    moose.vec('/model/chem/dend/IRSpGr/curv_IRSp53').n)
        allres.append(res)

    print("== Time taken %g s" % (time.time() - t0))

    # These values are created by GSL based solver. Boost solvers generates
    # slightly different values.
    gslExpected = [[1.43312300e+04, 3.59179272e-10, 7.90030831e-09, 1.43312300e+04, 0.00000000e+00],
        [1.43312300e+04, 5.85016305e-08, 6.01850896e-07, 1.43312300e+04, 0.00000000e+00],
        [1.43312300e+04, 1.88782106e-01, 1.78215231e+00, 1.43292591e+04, 0.00000000e+00]]
    gslExpected = np.array(gslExpected)

    allres = np.array(allres)
    # Compare the last value. When using Boost based solvers these numbers can
    # differ a bit but not more than 1%ii.
    assert np.allclose(gslExpected, allres, rtol=1e-2, atol=1e-3), (allres, gslExpected)
    print('done')

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import print_function, division

# Description: class NeuroML for loading NeuroML from single file into MOOSE
# Version 1.0 by Aditya Gilra, NCBS, Bangalore, India, 2011 for serial MOOSE
# Version 1.5 by Niraj Dudani, NCBS, Bangalore, India, 2012, ported to parallel MOOSE
# Version 1.6 by Aditya Gilra, NCBS, Bangalore, India, 2012, further changes for parallel MOOSE
# Maintainer: Dilawar Singh, dilawars@ncbs.res.in 
#  - python3 related fixes.

"""
NeuroML.py is the preferred interface to read NeuroML files.

Instantiate NeuroML class, and thence use method:

readNeuroMLFromFile(...) to load NeuroML from a file:

(a) the file could contain all required levels 1, 2 and 3 - Morph, Channel and Network;

OR

(b) the file could have only L3 (network) with L2 (channels/synapses) and L1
(cells) spread over multiple files;
 these multiple files should be in the same or lower-level directory
 named as <chan/syn_name>.xml or <cell_name>.xml or <cell_name>.morph.xml
 (essentially as generated by neuroConstruct's export).

OR

(c) the file could contain only L2 (MorphML) or only L1 (ChannelML),
 in which case the loader reads them into /library or at the path passed in.

OR

(d) your lower level L1 and L2 xml files could be scattered in non-hierarchical directories,
 then you can use this reader to load the ChannelML files individually into /library,
 then this reader to load MorphML files into /library.
 Store the cellDict returned for each cell, and combine them into a larger cellsDict as below,
 and finally call this reader on the NetworkML file passing the cellsDict.
 [OR you can use the separate Channel, Morph and NetworkML loaders in moose.neuroml.<...>
 to load each level files individually i.e. first load ChannelML files into /library,
 then cells from MorphML and, finally the NetworkML.]

For testing, you can also call this from the command line with a neuroML file as argument.
However, the new relative import .ChannelML breaks calling it from commandline,
 so have to do it on python / ipython terminal:
In [1]: import moose
In [2]: import moose.neuroml
In [3]: moose.neuroml.loadNeuroML_L123('Generated.net.xml')
"""

import moose
import moose.utils as mu
from xml.etree import cElementTree as ET
from moose.neuroml.ChannelML import ChannelML
from moose.neuroml.MorphML import MorphML
from moose.neuroml.NetworkML import NetworkML

from moose.neuroml.utils import *

import sys
from os import path

class NeuroML():

    def __init__(self):
        pass

    def readNeuroMLFromFile(self, filename, params={}, cellsDict={}):
        """
        For the format of params required to tweak what cells are loaded,
         refer to the doc string of NetworkML.readNetworkMLFromFile().
        Returns (populationDict,projectionDict),
         see doc string of NetworkML.readNetworkML() for details.
        """
        mu.info("Loading neuroml file %s " % filename)
        moose.Neutral('/library') # creates /library in MOOSE tree; elif present, wraps
        tree = ET.parse(filename)
        root_element = tree.getroot()

        # if model_path is given in params, use it else use the directory of NML
        # as model_dir.
        self.model_dir = params.get('model_dir', path.dirname(path.abspath(filename)))

        if 'lengthUnits' in list(root_element.attrib.keys()):
            self.lengthUnits = root_element.attrib['lengthUnits']
        else:
            self.lengthUnits = 'micrometer'

        ## lots of gymnastics to check if temperature meta tag is present
        self.temperature = CELSIUS_default # gets replaced below if tag for temperature is present
        self.temperature_default = True
        for meta_property in root_element.findall('.//{'+meta_ns+'}property'):
            ## tag can be an attrib or an element
            if 'tag' in list(meta_property.attrib.keys()): # tag is an attrib
                tagname = meta_property.attrib['tag']
                if 'temperature' in tagname:
                    self.temperature = float(meta_property.attrib['value'])
                    self.temperature_default = False
            else: # tag is a separate element
                tag = meta_property.find('.//{'+meta_ns+'}tag')
                tagname = tag.text
                if 'temperature' in tagname:
                    ## value can be a tag or an element
                    if 'value' in list(tag.attrib.keys()): # value is an attrib
                        self.temperature = float(tag.attrib['value'])
                        self.temperature_default = False
                    else: # value is a separate element
                        self.temperature = float(tag.find('.//{'+meta_ns+'}value').text)
                        self.temperature_default = False
        if self.temperature_default:
            mu.info("Using default temperature of %s degree Celsius" % self.temperature)
        self.nml_params = {
                'temperature':self.temperature,
                'model_dir':self.model_dir,
        }


        mu.debug("Loading channels and synapses into MOOSE /library ...")
        cmlR = ChannelML(self.nml_params)
        for channels in root_element.findall('.//{'+neuroml_ns+'}channels'):
            self.channelUnits = channels.attrib['units']
            for channel in channels.findall('.//{'+cml_ns+'}channel_type'):
                ## ideally I should read in extra params
                ## from within the channel_type element and put those in also.
                ## Global params should override local ones.
                cmlR.readChannelML(channel,params=params,units=self.channelUnits)
            for synapse in channels.findall('.//{'+cml_ns+'}synapse_type'):
                cmlR.readSynapseML(synapse,units=self.channelUnits)
            for ionConc in channels.findall('.//{'+cml_ns+'}ion_concentration'):
                cmlR.readIonConcML(ionConc,units=self.channelUnits)

        mu.debug("Loading cell definitions into MOOSE /library ...")
        mmlR = MorphML(self.nml_params)
        self.cellsDict = cellsDict
        for cells in root_element.findall('.//{'+neuroml_ns+'}cells'):
            for cell in cells.findall('.//{'+neuroml_ns+'}cell'):
                cellDict = mmlR.readMorphML(cell,params=params,lengthUnits=self.lengthUnits)
                self.cellsDict.update(cellDict)

        ## check if there are populations in this NML files,
        ## if not, it's a MorphML or ChannelML file, not NetworkML, so skip.
        if root_element.find('.//{'+neuroml_ns+'}populations') is None \
            and root_element.find('.//{'+nml_ns+'}populations') is None:
            return (self.cellsDict,'no populations (L3 NetworkML) found.')
        else:
            mu.debug("Loading individual cells into MOOSE root ... ")
            nmlR = NetworkML(self.nml_params)
            return nmlR.readNetworkML(root_element,self.cellsDict,\
                    params=params,lengthUnits=self.lengthUnits)
        ## cellsDict = { cellname: (segDict, cableDict), ... } # multiple cells
        ## where segDict = { segid1 : [ segname,(proximalx,proximaly,proximalz),
        ##     (distalx,distaly,distalz),diameter,length,[potential_syn1, ... ] ] , ... }
        ## segname is "<name>_<segid>" because 1) guarantees uniqueness,
        ##     & 2) later scripts obtain segid from the compartment's name!
        ## and cableDict = { cablegroupname : [campartment1name, compartment2name, ... ], ... }
        self.cellsDict = nmlR.cellSegmentDict

def loadNeuroML_L123(filename):
    neuromlR = NeuroML()
    return neuromlR.readNeuroMLFromFile(filename)

if __name__ == "__main__":
    if len(sys.argv)<2:
        mu.error("You need to specify the neuroml filename.")
        sys.exit(1)
    print(loadNeuroML_L123(sys.argv[1]))



# -*- coding: utf-8 -*-
# A library with some print functions. Very useful during development.

from __future__ import print_function, division, absolute_import

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2013, NCBS Bangalore"
__credits__          = ["NCBS Bangalore", "Bhalla Lab"]
__license__          = "GPL"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import moose

HEADER = '\033[95m'
OKBLUE = '\033[94m'
OKGREEN = '\033[92m'
WARNING = '\033[90m'
ERR = '\033[31m'
ENDC = '\033[0m'
RED = ERR
WARN = WARNING
INFO = OKGREEN
TODO = OKBLUE
DEBUG = HEADER
ERROR = ERR

prefixDict = dict(
    ERR = ERR
    , ERROR = ERR
    , WARN = WARN
    , FATAL = ERR
    , INFO = INFO
    , TODO = TODO
    , NOTE = HEADER
    , DEBUG = DEBUG
    , STEP = INFO
    , FAIL = ERR
    , FAILED = ERR
    )

def colored(msg, label="INFO") :
    """
    Return a colored string. Formatting is optional.

    At each ` we toggle the color.

    """
    global prefixDict
    if label in prefixDict :
        color = prefixDict[label]
    else :
        color = ""
    txt = ''
    newMsg = msg.split('`')
    i = 0
    for m in newMsg:
        if i % 2 == 0:
            txt += color + m
        else:
            txt += ENDC + m
        i += 1
    return "{0} {1}".format(txt, ENDC)

def cl(msg, label="INFO"):
    return colored(msg, label)

def dump(label, msg, frame=None, exception=None):
    ''' If msg is a list then first msg in list is the main message. Rest are
    sub message which should be printed prefixed by \n\t.
    '''

    prefix = '[{0}] '.format(label)

    '''
    # Enable it if you want indented messages
    import inspect
    stackLength = len(inspect.stack()) - 1
    if stackLength == 1:
        prefix = '\n[{}] '.format(label)
    else:
        prefix = ' '.join(['' for x in range(stackLength)])
    '''

    if type(msg) == list:
        if len(msg) > 1:
            msg = [msg[0]] + ["`|- {0}`".format(x) for x in msg[1:]]
        msg ="\n\t".join(msg)


    if not frame :
        print((prefix+"{0}".format(colored(msg,label))))
    else :
        filename = frame.f_code.co_filename
        filename = "/".join(filename.split("/")[-2:])
        print((prefix+"@{0}:{1} {2}".format(filename, frame.f_lineno, colored(msg, label))))
    if exception:
        print((" [Expcetion] {0}".format(exception)))

def info(msg):
    dump("INFO", msg)

def warn(msg):
    dump("WARN", msg)

def error(msg):
    dump("ERROR", msg)

def fatal(msg):
    dump("FATAL", msg)
    raise Exception(msg)

def debug(msg): dump("DEBUG", msg)

def log(label, msg): dump(label, msg)

## Following prints the elements in model
def modelInfo(path = '/##', **kwargs):
    """Generate the list of all available moose-elements in model
    """
    info("Couting elements in model under %s" % path)
    msg = []
    types = [ "Table", "Table2", "Compartment", "Pool", "BufPool", "Enz", "Reac" ]
    for t in types:
        paths = moose.wildcardFind("{}[TYPE={}]".format(path, t))
        if len(paths) > 0:
            msg.append("{:>20} : {}".format(t, len(paths)))
    return "\n".join(msg)



# -*- coding: utf-8 -*-
## Aditya Gilra, NCBS, Bangalore, 2012
## Dilawar Singh, NCBS, 2015

import os
os.environ['NUMPTHREADS'] = '1'
import sys

import moose
from moose.utils import *
import moose.utils as mu
from . import count
import numpy as np

from moose.neuroml.NeuroML import NeuroML

import unittest

simdt = 10e-6 # s
plotdt = 10e-6 # s
runtime = 0.19 # s

def loadGran98NeuroML_L123(filename):
    neuromlR = NeuroML()
    populationDict, projectionDict = \
        neuromlR.readNeuroMLFromFile(filename)
    soma_path = populationDict['CA1group'][1][0].path+'/Seg0_soma_0_0'
    somaVm = setupTable('somaVm',moose.Compartment(soma_path),'Vm')
    soma = moose.Compartment(soma_path)
    moose.reinit()
    moose.start(runtime)
    tvec = np.arange(0.0,runtime,simdt)
    res =  count.spike_train_simple_stat( somaVm.vector )
    return res['number of spikes']

if __name__ == "__main__":
    if len(sys.argv)<2:
        filename = "CA1soma.net.xml"
    else:
        filename = sys.argv[1]
    loadGran98NeuroML_L123(filename)





# -*- coding: utf-8 -*-
import moose
import numpy as np

def test_vec():
    foo = moose.Pool('/foo1', 500)
    #  bar = moose.vec('/foo1')
    bar = moose.vec(foo)
    assert len(bar) == 500, len(bar)
    for i in range(len(bar)):
        print(bar[i])

def test_vec2():
    iaf = moose.vec('/iaf', n=10, dtype='IntFire')

    print(iaf, 111)
    iaf.Vm = np.arange(0, 10, 1.0)
    print(iaf.Vm)
    print("All done")
    assert iaf[5].Vm == 5, iaf[5].Vm
    print(iaf.Vm)

def test_vec3():
    print("test vec3")
    foo = moose.Pool('/foo3', 500)
    foo.vec.concInit = 0.123
    print(foo.vec)
    assert foo.concInit == 0.123, foo.concInit
    assert np.allclose(foo.vec.concInit, [0.123]*500)


if __name__ == '__main__':
    test_vec()
    test_vec2()
    test_vec3()
